/********************************

 Based on 

 Simple config file class.

 Author: Ulrich Clanget
 Copying: Freeware

********************************/

#ifndef CONFIGGROUP_H
#define CONFIGGROUP_H

#include <qstring.h>
#include <qmap.h>
#include <qstringlist.h>

// group map type
typedef QMap<QString, QString> GroupMap;


class ConfigGroup
{
  friend class ConfigFile;

 public:

  ConfigGroup(const QString& groupname);
  ConfigGroup();

  void setName(const QString& groupname);
  const QString& name() const { return groupname; }

  const QString& readEntry(const QString& key, const QString& default_value = QString::null);
  void writeEntry(const QString& key, const QString& value);

  bool isEmpty() const { return map.isEmpty(); }

 private:

  QString groupname;
  GroupMap map;
};

#endif

/* ConfigFile.h */

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

//#include "ConfigGroup.h"

//#include <qstringlist.h>

typedef QMap<QString, ConfigGroup*> ConfigMap;

class ConfigFile
{
 public:

  ConfigFile();
  ConfigFile(const QString& filename);
  ~ConfigFile();

  // clean up
  void clear();

  // file
  void setName(const QString& filename);
  const QString& name() const { return fname; }
  void write();
  void read();

  // current group
  void setGroup(const QString& group);
  const QString& group() const;

  // group info
  unsigned int numGroups() const { return map.count(); }
  QStringList groupList();

  // write entries
  void writeEntry(const QString& key, const QString& value);
  void writeEntry(const QString& key, int value);
  void writeEntry(const QString& key, double value);

  // read entries
  const QString& readEntry(const QString& key, const QString& default_value);
  int readEntry(const QString& key, int default_value);
  double readEntry(const QString& key, double default_value);

 private:

  QString fname;
  ConfigMap map;
  ConfigGroup* currentgroup;
};

#endif
