/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "datacontainer.h" 
#include <iostream.h> 
#include "qt_oraconnect.h"
#include <qheader.h>
#include <qstring.h>

DataContainer::DataContainer(qt_oraconnect * cObj, const QString& sql)
//DataContainer::DataContainer(qt_oraconnect * cObj, const char * sql)
	: DataSource(0),sql(QString::null),originalSQL(QString::null),
	  logMode(false),noInitQuery(false)
	//: DataSource(0),sql(0),originalSQL(0)
{
	setConnObj(cObj);
	if (sql)
	{
		setSQL(sql);
		ParseSQL();
	}
}

DataContainer::~DataContainer()
{
#ifdef DESTRACE
	cerr << "DataContainer destructor" << endl;
#endif
	//delete [] sql;
	//sql=0;
	//delete [] originalSQL;
	//originalSQL=0;
}

const QString& DataContainer::getSQL() //const
//const char* DataContainer::getSQL() //const
{
	QString tSTR=select+from;
	if (!where.isEmpty())
	{
		tSTR=tSTR+where;
		if (!filter.isEmpty())
			tSTR=tSTR+" AND "+filter;
	}
	else
		if (!filter.isEmpty())
                        tSTR=tSTR+" WHERE "+filter;

	if (!sort.isEmpty())
		tSTR=tSTR+" ORDER BY "+sort;
	else
		tSTR=tSTR+orderby;

	//QString tSTR=select+from+where+filter+orderby+sort;
	//sql = new char[strlen(tSTR)+1];
	//strcpy(sql,tSTR);
	sql=tSTR;
	return sql;
}

const QString& DataContainer::getOriginalSQL() const
//const char* DataContainer::getOriginalSQL() const
{
	return originalSQL;
}

void DataContainer::setSQL(const QString& sql)
//void DataContainer::setSQL(const char * sql)
{
        //originalSQL= new char[strlen(sql)+1];
        //strcpy(originalSQL,sql); 
	originalSQL=sql;
}

void DataContainer::setConnObj(qt_oraconnect * cObj)
{
	DataSource = cObj;
}

qt_oraconnect * DataContainer::getConnObj()
{
        return DataSource;
}

void DataContainer::ParseSQL()
{
	QString tSQL(originalSQL);

	// Extract select clause
	int select_end_pos=tSQL.find(" from ",0,false);
	if (select_end_pos==-1) return;
	else select=tSQL.left(select_end_pos);	

	// Extract from clause
	int from_end_pos=tSQL.find(" where ",select_end_pos,false);
	if (from_end_pos==-1)
	{
		// OK so there is no 'where' but there may be an order by
		from_end_pos=tSQL.find(" order by ",0,false);
		if (from_end_pos==-1)
		{
			// THere is no 'order by' either so the end of the 
			// from is the end of the string
			from_end_pos=tSQL.length();
		}
	}
	from=tSQL.mid(select_end_pos,from_end_pos-select_end_pos);

	// Extract where clause
	int where_end_pos=tSQL.find(" order by ",from_end_pos,false);
	if (where_end_pos==-1)
	{
		// There is no order by so the end of the 
                // where is the end of the string
		where_end_pos=tSQL.length();
	}	
	where=tSQL.mid(from_end_pos,where_end_pos-from_end_pos);

	// Extract order by clause
	if ((uint)where_end_pos<tSQL.length())
	{
		// There is an order by clause
		// Lets extract it
		orderby=tSQL.right(tSQL.length()-where_end_pos); 
	}
}

void DataContainer::addToColList(const QString col)
{
	columns << col;
}

void DataContainer::clearColList()
{
	columns.clear();
}
	
const QStringList& DataContainer::getColList() const 
{
	return columns;
}

const QString& DataContainer::getSort() const
{
	return sort;
}

void DataContainer::setSort(const QString& s)
{
	sort=s;
}

const QString& DataContainer::getFilter() const
{
        return filter;
}

void DataContainer::setFilter(const QString& f)
{
        filter=f;
}

const bool DataContainer::getLogMode() const
{
	return(logMode);
}

void DataContainer::setLogMode(const bool lm)
{
	logMode=lm;
}

const bool& DataContainer::getNoInitQuery() const
{
	return(noInitQuery);
}

void DataContainer::setNoInitQuery(const bool niq)
{
	noInitQuery=niq;
}
