/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qstring.h>
#include "row.h"
#include "dbtab.h"
#include "dbform.h"
#include "griddatadisplay.h"
#include "formdatadisplay.h"
#include "dbobjects.h"
#include "w_dba_tables.h"
#include "w_dba_indexes.h"
#include "w_dba_views.h"
#include "w_dba_constraints.h"
#include "w_dba_constraints.h"
#include "w_dba_clusters.h"
#include "w_dba_sequences.h"
#include "w_dba_segments.h"
#include "w_dba_procedures.h"
#include "w_dba_functions.h"
#include "w_dba_packages.h"
#include "w_dba_triggers.h"
#include "w_dba_synonyms.h"
#include "w_dba_db_links.h"
#include "w_dba_objects.h"
#include "w_dba_outlines.h"
#include "w_dba_tab_partitions.h"
#include "w_dba_lobs.h"

#include "mdi.h"
#include "dblist.h"
#include "objparm.h"
#include "sqlcache.h"
#include "qt_oraconnect.h"
#include <qmessagebox.h>
#include <qobjcoll.h>
#include <qtoolbutton.h>

DBObjects::DBObjects(const QString& un, QWidget * parent, const QString& name)
//DBObjects::DBObjects(const QString& un, QWidget * parent, const char * name)
	: ListView(parent,name), username(un)
{
	setupObjectTypeList();
}

void DBObjects::setupObjectTypeList()
{
	setColumnAlignment(1,AlignRight);
	setColumnAlignment(2,AlignRight);

	addColumn("Object type");
	addColumn("Total number");
	addColumn("Total size [MB]");
	setSorting(0);
	setAllColumnsShowFocus(TRUE);
	setColumnWidthMode(0,QListView::Manual);
	setColumnWidthMode(1,QListView::Manual);
	setColumnWidthMode(2,QListView::Manual);
	setColumnWidth(0,180);
	setColumnWidth(1,180);
	setColumnWidth(2,180);

	row *r = new row(this);
	r->createCells(3);
	r->setText(0,"Tables");
	tables=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Views");
	views=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Indexes");
	indexes=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Clusters");
	clusters=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Sequences");
	sequences=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Procedures");
	procedures=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Functions");
	functions=r;	

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Synonyms");
	synonyms=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Triggers");
	triggers=r;


	r = new row(this);
	r->createCells(3);
	r->setText(0,"Constraints");
	constraints=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Jobs");
	jobs=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Packages");
	packages=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Segments");
	segments=r;
	r = new row(this);
	r->createCells(3);
	r->setText(0,"Database links");
	dblinks=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Snapshots");
	snapshots=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Snapshot logs");
	snapshot_logs=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Objects");
	objects=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Source lines");
	source_lines=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Extents");
	extents=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Blocks");
	blocks=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Columns");
	columns=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Types");
	types=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Libraries");
	libraries=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Outlines");
	outlines=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Partitions (Table)");
	tab_partitions=r;

	r = new row(this);
	r->createCells(3);
	r->setText(0,"Lobs");
	lobs=r;

	setSelected(firstChild(),true); 

	connect(this, SIGNAL(doubleClicked(row*)), this, SLOT(callObjectDetails(row *)));
	connect(this, SIGNAL(returnPressed(row*)), this, SLOT(callObjectDetails(row *)));
}

void DBObjects::callObjectDetails(row * clickedRow)
{
	QString objType = clickedRow->getCellbyName("Object type");

	if (objType=="Tables") 		callTables();
	else if (objType=="Indexes")	callIndexes();
	else if (objType=="Objects")	callObjects();
	else if (objType=="Views")	callViews();
	else if (objType=="Sequences")	callSequences();
	else if (objType=="Segments")	callSegments();
	else if (objType=="Constraints")	callConstraints();
	else if (objType=="Clusters")	callClusters();
	else if (objType=="Procedures")	callProcedures();
	else if (objType=="Functions")	callFunctions();
	else if (objType=="Packages")	callPackages();
	else if (objType=="Triggers")	callTriggers();
	else if (objType=="Synonyms")	callSynonyms();
	else if (objType=="Jobs")	callJobs();
	else if (objType=="Database links")	callDBLinks();
	else if (objType=="Snapshots")	callSnapshots();
	else if (objType=="Snapshot logs")	callSnapshotLogs();
	else if (objType=="Libraries")	callLibraries();
	else if (objType=="Types")	callTypes();
	else if (objType=="Outlines")	callOutlines();
	else if (objType=="Partitions (Table)")	callTabPartitions();
	else if (objType=="Lobs")	callLobs();
}

void DBObjects::callTables()
{
	w_dba_tables *w=0;
	if (username)
        	w = new w_dba_tables(username);
	else
		w = new w_dba_tables();
        w->show();
}

void DBObjects::callOutlines()
{
        w_dba_outlines *w=0;
        if (username)
                w = new w_dba_outlines(username);
        else
                w = new w_dba_outlines();
        w->show();
}

void DBObjects::callTabPartitions()
{
        w_dba_tab_partitions *w=0;
        if (username)
                w = new w_dba_tab_partitions(username);
        else
                w = new w_dba_tab_partitions();
        w->show();
}

void DBObjects::callLobs()
{
        w_dba_lobs *w=0;
        if (username)
                w = new w_dba_lobs(username);
        else
                w = new w_dba_lobs();
        w->show();
}

void DBObjects::callViews()
{
        w_dba_views *w=0;
	if (username)
        	w = new w_dba_views(username);
	else
		w = new w_dba_views();
        w->show();
}

void DBObjects::callIndexes()
{
        w_dba_indexes *w = 0;
	if (username)
		w=new w_dba_indexes(username);
	else
		w=new w_dba_indexes();
        w->show();
}

void DBObjects::callObjects()
{
        w_dba_objects *w = 0;
        if (username)
                w=new w_dba_objects(username);
        else
                w=new w_dba_objects();
        w->show();
}

void DBObjects::callSegments()
{
        w_dba_segments *w = 0;
        if (username)
                w=new w_dba_segments(username);
        else
                w=new w_dba_segments();
        w->show();
}

void DBObjects::callSequences()
{
        w_dba_sequences *w = 0;
        if (username)
                w=new w_dba_sequences(username);
        else
                w=new w_dba_sequences();

        w->show();
}

void DBObjects::callConstraints()
{
        w_dba_constraints *w = 0;
        if (username)
                w=new w_dba_constraints(username);
        else
                w=new w_dba_constraints();

        w->show();
}

void DBObjects::callClusters()
{
        w_dba_clusters *w = 0;
        if (username)
                w=new w_dba_clusters(username);
        else
                w=new w_dba_clusters();
        w->show();
}

void DBObjects::callProcedures()
{
        w_dba_procedures *w = 0;
        if (username)
                w=new w_dba_procedures(username);
        else
                w=new w_dba_procedures();

        w->show();
}

void DBObjects::callFunctions()
{
        w_dba_functions *w = 0;
        if (username)
                w=new w_dba_functions(username);
        else
                w=new w_dba_functions();
        w->show();
}

void DBObjects::callPackages()
{
        w_dba_packages *w = 0;
        if (username)
                w=new w_dba_packages(username);
        else
                w=new w_dba_packages();

        w->show();
}

void DBObjects::callTriggers()
{
        w_dba_triggers *w = 0;
        if (username)
                w=new w_dba_triggers(username);
        else
                w=new w_dba_triggers();

        w->show();
}

void DBObjects::callSynonyms()
{
        w_dba_synonyms *w = 0;
        if (username)
                w=new w_dba_synonyms(username);
        else
                w=new w_dba_synonyms();

        w->show();
}

void DBObjects::callJobs()
{
        DBList *w = 0;
        if (username)
		w=new DBList(username,"dba_jobs_per_user",0,"DBA_JOBS for user "+username);
        else
                w=new DBList("dba_jobs",0,"DBA_JOBS");
        w->show();
}

void DBObjects::callDBLinks()
{
        w_dba_db_links *w = 0;
        if (username)
                w=new w_dba_db_links(username);
        else
                w=new w_dba_db_links();

        w->show();
}

void DBObjects::callSnapshots()
{
        DBList *w = 0;
        if (username)
		w=new DBList(username,"dba_snapshots_per_user",0,"DBA_SNAPSHOTS for user "+username);
        else
                w=new DBList("dba_snapshots",0,"DBA_SNAPSHOTS");
        w->show();
}

void DBObjects::callSnapshotLogs()
{
        DBList *w = 0;
        if (username)
                w=new DBList(username,"dba_snapshot_logs_per_user",0,"DBA_SNAPSHOT_LOGS for user "+username);
        else
                w=new DBList("dba_snapshot_logs",0,"DBA_SNAPSHOT_LOGS");
        w->show();
}

void DBObjects::callTypes()
{
        DBList *w = 0;
        if (username)
                w=new DBList(username,"dba_types_per_user",0,"DBA_TYPES for user "+username);
        else
                w=new DBList("dba_types",0,"DBA_TYPES");
        w->show();
}

void DBObjects::callLibraries()
{
        DBList *w = 0;
        if (username)
                w=new DBList(username,"dba_libraries_per_user",0,"DBA_LIBRARIES for user "+username);
        else
                w=new DBList("dba_libraries",0,"DBA_LIBRARIES");
        w->show();
}

void DBObjects::calcCurNoOfObjects()
{
	calcNoOfObjects((row *)(selectedItem()));
	((row *)(selectedItem()))->repaint();
}

void DBObjects::calcAllNoOfObjects()
{
	QListViewItemIterator it(this);

        for (;it.current(); ++it)
        {
			calcNoOfObjects((row *)(it.current()));
			((row *)(it.current()))->repaint();
        }
}

void DBObjects::calcNoOfObjects(row * r)
{
        QString objType = r->getCellbyName("Object type");

        if (objType=="Tables")          calcNoOfTables();
        else if (objType=="Views")      calcNoOfViews();
        else if (objType=="Indexes")    calcNoOfIndexes();
        else if (objType=="Objects")    calcNoOfDBObjects();
        else if (objType=="Segments")   calcNoOfSegments();
        else if (objType=="Sequences")  calcNoOfSequences();
        else if (objType=="Constraints")        calcNoOfConstraints();
        else if (objType=="Clusters")   calcNoOfClusters();
        else if (objType=="Procedures") calcNoOfProcedures();
        else if (objType=="Functions")  calcNoOfFunctions();
        else if (objType=="Packages")   calcNoOfPackages();
        else if (objType=="Triggers")   calcNoOfTriggers();
        else if (objType=="Synonyms")   calcNoOfSynonyms();
        else if (objType=="Jobs")       calcNoOfJobs();
        else if (objType=="Database links")     calcNoOfDBLinks();
        else if (objType=="Snapshots")  calcNoOfSnapshots();
        else if (objType=="Snapshot logs")      calcNoOfSnapshotLogs();
        else if (objType=="Types")      calcNoOfTypes();
        else if (objType=="Libraries")  calcNoOfLibraries();
        else if (objType=="Blocks")  calcNoOfBlocks();
        else if (objType=="Columns")  calcNoOfColumns();
        else if (objType=="Extents")  calcNoOfExtents();
        else if (objType=="Source lines")  calcNoOfSourceLines();
        else if (objType=="Outlines")  calcNoOfOutlines();
        else if (objType=="Partitions (Table)")  calcNoOfTabPartitions();
        else if (objType=="Lobs")  calcNoOfLobs();
}

void DBObjects::calcNoOfTables()
{
	try
        {
		if (username)
		{
	        	qt_OraParm op(1);
        		op.addParm(username);
        		gConnObj->open(gSQLCache->getSQL("calc_table_number_per_user"),op);
		}
		else
			gConnObj->open(gSQLCache->getSQL("calc_table_number"));
        	gConnObj->fetch();
        	tables->setText(1,gConnObj->getCol(0));
        	gConnObj->close();
	}
	catch (oerr& e)
	{
		QMessageBox::critical( 0, "Database error during table counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void DBObjects::calcNoOfViews()
{
	try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_view_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_view_number"));
                gConnObj->fetch();
                views->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during view counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }

}

void DBObjects::calcNoOfOutlines()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_outline_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_outline_number"));
                gConnObj->fetch();
                outlines->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during outline counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }

}

void DBObjects::calcNoOfTabPartitions()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_tab_partition_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_tab_partition_number"));
                gConnObj->fetch();
                tab_partitions->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during table partition counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }

}

void DBObjects::calcNoOfLobs()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_lob_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_lob_number"));
                gConnObj->fetch();
                lobs->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during lob counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }

}

void DBObjects::calcNoOfIndexes()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_index_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_index_number"));
                gConnObj->fetch();
                indexes->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during index counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfDBObjects()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_object_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_object_number"));
                gConnObj->fetch();
                objects->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during object counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }

}


void DBObjects::calcNoOfSegments()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_segment_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_segment_number"));
                gConnObj->fetch();
                segments->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during segment counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfSequences()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_sequence_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_sequence_number"));
                gConnObj->fetch();
                sequences->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during sequence counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfConstraints()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_constraint_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_constraint_number"));
                gConnObj->fetch();
                constraints->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during constraint counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfClusters()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_cluster_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_cluster_number"));
                gConnObj->fetch();
                clusters->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during cluster counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfProcedures()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_procedure_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_procedure_number"));
                gConnObj->fetch();
                procedures->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during procedure counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfFunctions()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_function_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_function_number"));
                gConnObj->fetch();
                functions->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during functions counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfPackages()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_package_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_package_number"));
                gConnObj->fetch();
                packages->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during package counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfTriggers()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_trigger_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_trigger_number"));
                gConnObj->fetch();
                triggers->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during trigger counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfSynonyms()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_synonym_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_synonym_number"));
                gConnObj->fetch();
                synonyms->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during synonym counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfJobs()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_job_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_job_number"));
                gConnObj->fetch();
                jobs->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during job counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfDBLinks()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_dblink_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_dblink_number"));
                gConnObj->fetch();
                dblinks->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during database link counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfSnapshots()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_snapshot_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_snapshot_number"));
                gConnObj->fetch();
                snapshots->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during snapshot counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfSnapshotLogs()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_snapshot_log_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_snapshot_log_number"));
                gConnObj->fetch();
                snapshot_logs->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during snapshot log counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfTypes()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_type_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_type_number"));
                gConnObj->fetch();
                types->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during type counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfLibraries()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_library_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_library_number"));
                gConnObj->fetch();
                libraries->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during library counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfColumns()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_column_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_column_number"));
                gConnObj->fetch();
                columns->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during column counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfExtents()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_extent_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_extent_number"));
                gConnObj->fetch();
                extents->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during extent counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfSourceLines()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_source_line_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_source_line_number"));
                gConnObj->fetch();
                source_lines->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during source line counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcNoOfBlocks()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(1);
                        op.addParm(username);
                        gConnObj->open(gSQLCache->getSQL("calc_block_number_per_user"),op);
                }
                else
                        gConnObj->open(gSQLCache->getSQL("calc_block_number"));
                gConnObj->fetch();
                blocks->setText(1,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during block counting", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

//==================================
// Calculate sizes

void DBObjects::calcCurSizeOfObjects()
{
	calcSizeOfObjects((row *)(selectedItem()));
	((row *)(selectedItem()))->repaint();
}

void DBObjects::calcAllSizeOfObjects()
{
	QListViewItemIterator it(this);

        for (;it.current(); ++it)
        {
		calcSizeOfObjects((row *)(it.current()));
		((row *)(it.current()))->repaint();
        }
}

void DBObjects::calcSizeOfObjects(row * r)
{
        QString objType = r->getCellbyName("Object type");

        if (objType=="Tables")          calcSizeOfTables();
        else if (objType=="Views")      calcSizeOfViews();
        else if (objType=="Indexes")    calcSizeOfIndexes();
        else if (objType=="Objects")    calcSizeOfDBObjects();
        else if (objType=="Segments")   calcSizeOfSegments();
        else if (objType=="Sequences")  calcSizeOfSequences();
        else if (objType=="Constraints")        calcSizeOfConstraints();
        else if (objType=="Clusters")   calcSizeOfClusters();
        else if (objType=="Procedures") calcSizeOfProcedures();
        else if (objType=="Functions")  calcSizeOfFunctions();
        else if (objType=="Packages")   calcSizeOfPackages();
        else if (objType=="Triggers")   calcSizeOfTriggers();
        else if (objType=="Synonyms")   calcSizeOfSynonyms();
        else if (objType=="Jobs")       calcSizeOfJobs();
        else if (objType=="Database links")     calcSizeOfDBLinks();
        else if (objType=="Snapshots")  calcSizeOfSnapshots();
        else if (objType=="Snapshot logs")      calcSizeOfSnapshotLogs();
        else if (objType=="Types")      calcSizeOfTypes();
        else if (objType=="Libraries")  calcSizeOfLibraries();
        else if (objType=="Blocks")  calcSizeOfBlocks();
        else if (objType=="Columns")  calcSizeOfColumns();
        else if (objType=="Extents")  calcSizeOfExtents();
        else if (objType=="Source lines")  calcSizeOfSourceLines();
        else if (objType=="Outlines")  calcSizeOfOutlines();
        else if (objType=="Partitions (Table)")  calcSizeOfTabPartitions();
        else if (objType=="Lobs")  calcSizeOfLobs();
}

void DBObjects::calcSizeOfTables()
{
	try
	{
		if (username)
		{
        		qt_OraParm op(2);
        		op.addParm(username);
        		op.addParm("TABLE");
        		gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
		}
		else
		{
			qt_OraParm op(1);
        		op.addParm("TABLE");
        		gConnObj->open(gSQLCache->getSQL("calc_segment_size"),op);
		}

        	gConnObj->fetch();
        	tables->setText(2,gConnObj->getCol(0));
        	gConnObj->close();
	}
	catch (oerr& e)
	{
		QMessageBox::critical( 0, "Database error during table size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void DBObjects::calcSizeOfViews()
{
        views->setText(2,"N/A");
}

void DBObjects::calcSizeOfOutlines()
{
        outlines->setText(2,"N/A");
}

void DBObjects::calcSizeOfTabPartitions()
{
        tab_partitions->setText(2,"N/A");
}

void DBObjects::calcSizeOfLobs()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(2);
                        op.addParm(username);
                        op.addParm("LOBSEGMENT");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
                }
                else
                {
                        qt_OraParm op(1);
                        op.addParm("LOBSEGMENT");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size"),op);
                }

                gConnObj->fetch();
                lobs->setText(2,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during lob size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok |QMessageBox::Default,0,0);
        }
}

void DBObjects::calcSizeOfIndexes()
{
	try
	{
		if (username)
		{
        		qt_OraParm op(2);
        		op.addParm(username);
        		op.addParm("INDEX");
        		gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
		}
		else
		{
			qt_OraParm op(1);
        		op.addParm("INDEX");
        		gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
		}
        	gConnObj->fetch();
        	indexes->setText(2,gConnObj->getCol(0));
        	gConnObj->close();
	}
	catch (oerr& e)
	{
		QMessageBox::critical( 0, "Database error during index size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void DBObjects::calcSizeOfDBObjects()
{
        objects->setText(2,"N/A");
}


void DBObjects::calcSizeOfSegments()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(2);
                        op.addParm(username);
                        op.addParm("%");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
                }
                else
                {
                        qt_OraParm op(1);
                        op.addParm("%");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size"),op);
                }

                gConnObj->fetch();
                segments->setText(2,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during segment size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcSizeOfSequences()
{
        sequences->setText(2,"N/A");
}

void DBObjects::calcSizeOfConstraints()
{
        constraints->setText(2,"N/A");
}

void DBObjects::calcSizeOfClusters()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(2);
                        op.addParm(username);
                        op.addParm("CLUSTER");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
                }
                else
                {
                        qt_OraParm op(1);
                        op.addParm("CLUSTER");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size"),op);
                }

                gConnObj->fetch();
                clusters->setText(2,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during cluster size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcSizeOfProcedures()
{
        procedures->setText(2,"N/A");
}

void DBObjects::calcSizeOfFunctions()
{
        functions->setText(2,"N/A");
}

void DBObjects::calcSizeOfPackages()
{
        packages->setText(2,"N/A");
}

void DBObjects::calcSizeOfTriggers()
{
        triggers->setText(2,"N/A");
}

void DBObjects::calcSizeOfSynonyms()
{
        synonyms->setText(2,"N/A");
}

void DBObjects::calcSizeOfJobs()
{
        jobs->setText(2,"N/A");
}

void DBObjects::calcSizeOfDBLinks()
{
        dblinks->setText(2,"N/A");
}

void DBObjects::calcSizeOfSnapshots()
{
        try
        {
                if (username)
                {
                        qt_OraParm op(2);
                        op.addParm(username);
                        op.addParm("SNAPSHOT");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size_per_user"),op);
                }
                else
                {
                        qt_OraParm op(1);
                        op.addParm("SNAPSHOT");
                        gConnObj->open(gSQLCache->getSQL("calc_segment_size"),op);
                }

                gConnObj->fetch();
                snapshots->setText(2,gConnObj->getCol(0));
                gConnObj->close();
        }
        catch (oerr& e)
        {
                QMessageBox::critical( 0, "Database error during snapshot size calculation", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

void DBObjects::calcSizeOfSnapshotLogs()
{
        snapshot_logs->setText(2,"N/A");
}

void DBObjects::calcSizeOfTypes()
{
        types->setText(2,"N/A");
}

void DBObjects::calcSizeOfLibraries()
{
        libraries->setText(2,"N/A");
}

void DBObjects::calcSizeOfColumns()
{
        columns->setText(2,"N/A");
}

void DBObjects::calcSizeOfExtents()
{
        extents->setText(2,"N/A");
}

void DBObjects::calcSizeOfSourceLines()
{
        source_lines->setText(2,"N/A");
}

void DBObjects::calcSizeOfBlocks()
{
        blocks->setText(2,"N/A");
}

void DBObjects::calcAll()
{
	calcAllNoOfObjects();
	calcAllSizeOfObjects();
}

void DBObjects::calcCurSizeAndNumber()
{
	calcCurNoOfObjects(); 
	calcCurSizeOfObjects();
}
