/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#ifndef DBOBJECTS
#define DBOBJECTS

#include "listview.h"

class row;

class DBObjects: public ListView
{
	Q_OBJECT
public:
	//DBObjects(const QString& un=QString::null, QWidget * parent=0, const char * name=0);
	DBObjects(const QString& un=QString::null, QWidget * parent=0, 
		  const QString& name=QString::null);

private:
	void setupObjectTypeList();

	row *tables, *indexes, *clusters, *jobs, *synonyms, *sequences;
	row *views, *objects, *segments, *constraints, *procedures, *functions;
	row *packages, *triggers, *dblinks, *snapshots, *snapshot_logs;
	row *types, *libraries, *source_lines, *columns, *extents, *blocks;
	row *outlines, *tab_partitions, *subpartitions,*lobs;

	void callTables();
	void callViews();
	void callIndexes();
	void callObjects();
	void callSegments();
	void callSequences();
	void callConstraints();
	void callClusters();
	void callProcedures();
	void callFunctions();
	void callPackages();
	void callTriggers();
	void callSynonyms();
	void callJobs();
	void callDBLinks();
	void callSnapshots();
	void callSnapshotLogs();
	void callTypes();
	void callLibraries();
	void callOutlines();
	void callTabPartitions();
	void callLobs();
	

	void calcNoOfObjects(row * r);

        void calcNoOfTables();
        void calcNoOfViews();
        void calcNoOfIndexes();
        void calcNoOfDBObjects();
        void calcNoOfSegments();
        void calcNoOfSequences();
        void calcNoOfConstraints();
        void calcNoOfClusters();
        void calcNoOfProcedures();
        void calcNoOfFunctions();
        void calcNoOfPackages();
        void calcNoOfTriggers();
        void calcNoOfSynonyms();
        void calcNoOfJobs();
        void calcNoOfDBLinks();
        void calcNoOfSnapshots();
        void calcNoOfSnapshotLogs();
        void calcNoOfTypes();
        void calcNoOfLibraries();
	void calcNoOfColumns();
	void calcNoOfExtents();
	void calcNoOfSourceLines();
	void calcNoOfBlocks();
	void calcNoOfOutlines();
	void calcNoOfTabPartitions();
	void calcNoOfLobs();

        void calcSizeOfObjects(row * r);

        void calcSizeOfTables();
        void calcSizeOfViews();
        void calcSizeOfIndexes();
        void calcSizeOfDBObjects();
        void calcSizeOfSegments();
        void calcSizeOfSequences();
        void calcSizeOfConstraints();
        void calcSizeOfClusters();
        void calcSizeOfProcedures();
        void calcSizeOfFunctions();
        void calcSizeOfPackages();
        void calcSizeOfTriggers();
        void calcSizeOfSynonyms();
        void calcSizeOfJobs();
        void calcSizeOfDBLinks();
        void calcSizeOfSnapshots();
        void calcSizeOfSnapshotLogs();
        void calcSizeOfTypes();
        void calcSizeOfLibraries();
        void calcSizeOfColumns();
        void calcSizeOfExtents();
        void calcSizeOfSourceLines();
        void calcSizeOfBlocks();
        void calcSizeOfOutlines();
        void calcSizeOfTabPartitions();
        void calcSizeOfLobs();

	QString username;

private slots:
	void callObjectDetails(row * clickedRow);
	void calcCurNoOfObjects();
	void calcAllNoOfObjects();

	void calcCurSizeOfObjects();
	void calcAllSizeOfObjects();

	void calcAll();
	void calcCurSizeAndNumber();
};

#endif
