/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "dbtab.h" 
#include "mdi.h" 
#include "sqlcache.h" 
#include <iostream.h> 
#include <qmessagebox.h> 

DBTab::DBTab(QWidget * parent, const QString& name)
//DBTab::DBTab(QWidget * parent, const char * name)
	: DBSheet(parent, name) 
// Construct minimal object
{
	construct();
}

DBTab::DBTab(const ObjParm& parm, QWidget * parent, const QString& name)
//DBTab::DBTab(const ObjParm& parm, QWidget * parent, const char * name)
        : DBSheet(parm, parent, name) 
// Construct minimal object
{
        construct();
}

void DBTab::construct()
{
        setDD();
        connect((TabDataDisplay*)getDD(), SIGNAL(rightButtonPressed(const QPoint &)), this, SLOT(ContextMenu(const QPoint &)));
	// changing tabs should retrieve
	// This will also do the first retrieve after creation of the window
        connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT(retrieve()));
        connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT(adaptSheetTitle()));
        connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT(adaptActionMenu()));
	connect((TabDataDisplay*)getDD(), SIGNAL(SQLError(const QString&)), this, SLOT(SQLErrorHandler()));
}

DBTab::~DBTab()
{
#ifdef DESTRACE
	cerr << "DBTab destructor" << endl;
#endif
}

void DBTab::setDD()
{
	if (getDD())
	{
		removeDD();
	}
	TabDataDisplay* tdd = new TabDataDisplay(gConnObj,this,"dd_dbtab");
	
	assignDD(tdd);
	////getDD()->setNoInitQuery(gMDI->getNoInitQuery());
	setCentralWidget((TabDataDisplay*)getDD());
}

void DBTab::addTab(QWidget *widget, const QString& name)
{
	((TabDataDisplay*)getDD())->addTab(widget,name);
}

void DBTab::addGridTab(GridDataDisplay *grid, const QString& name, const QString& sqlname)
{
	((TabDataDisplay*)getDD())->addGridTab(grid,name);
	QString * queryName= new QString(sqlname);
	queryNames.insert(name,queryName);
}

void DBTab::addGridTab(QString sqlname, const QString& name, bool ana)
{
	if (!gSQLCache)
	{
		QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	}
        QString sql = gSQLCache->getSQL(sqlname);
        if (sql!="")
	{
		((TabDataDisplay*)getDD())->addGridTab(sql,name,ana);
		QString * queryName= new QString(sqlname);
		queryNames.insert(name,queryName);
	}
        else
	{
	        // Unwind the waitCursors that may have been set elsewhere
                while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
                QMessageBox::critical( 0, "SQL cache error", QString("The following SQL could not be found: ") + sqlname,QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void DBTab::addFormTab(FormDataDisplay *form, const QString& name, const QString& sqlname)
{
	((TabDataDisplay*)getDD())->addFormTab(form,name);
	QString * queryName= new QString(sqlname);
	queryNames.insert(name,queryName);
}

void DBTab::addFormTab(QString sqlname, const QString& name, int cols,int lw, int tw, QString SingleField,int sfw, int sfh)
{
        if (!gSQLCache)
        {
            QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
            return;
        }                                                                                                         
        QString sql = gSQLCache->getSQL(sqlname);
        if (sql!="")
		{
			((TabDataDisplay*)getDD())->addFormTab(sql,name,cols,lw,tw,SingleField,sfw,sfh);
			QString * queryName= new QString(sqlname);
			queryNames.insert(name,queryName);
		}
        else
                QMessageBox::critical( 0, "SQL cache error", QString("The following SQL could not be found: ") + sqlname,QMessageBox::Ok | QMessageBox::Default,0,0);
}

void DBTab::addMLETab(MLEDataDisplay *mle, const QString& name, const QString& sqlname)
{
        ((TabDataDisplay*)getDD())->addMLETab(mle,name);
        QString * queryName= new QString(sqlname);
        queryNames.insert(name,queryName);
}

void DBTab::addMLETab(QString sqlname, const QString& name, bool headers, QString line_sep, 
		      QString col_sep, QString head_sep)
{
        if (!gSQLCache)
        {
                QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
        }
        QString sql = gSQLCache->getSQL(sqlname);
        if (sql!="")
        {
                ((TabDataDisplay*)getDD())->addMLETab(sql,name,headers, line_sep, col_sep, head_sep);
                QString * queryName= new QString(sqlname);
                queryNames.insert(name,queryName);
        }
        else
                QMessageBox::critical( 0, "SQL cache error", QString("The following SQL could not be found: ") + sqlname,QMessageBox::Ok | QMessageBox::Default,0,0);
}

void DBTab::addTabTab(TabDataDisplay *tab, const QString& name)
{
	((TabDataDisplay*)getDD())->addTabTab(tab,name);
	//queryNames.insert(name,0);
}

const QString& DBTab::getQueryName() const
//const char * DBTab::getQueryName() const
{
	QString tabname=((TabDataDisplay*)getDD())->getCurrentTab();
	if (queryNames[tabname]) return(*queryNames[tabname]);
	else return (QString::null);
//	return(((TabDataDisplay*)getDD())->getCurrentTab());
}
