/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <qtextstream.h>
#include <qstring.h>
#include "gensql.h"
#include "griddatadisplay.h"
#include "objparm.h"
#include "sqlcache.h"
#include "row.h"
#include "qt_oraconnect.h"
#include <iostream.h>

extern QString gTNSAlias;

GenSQL::GenSQL()
{
}

GenSQL::~GenSQL()
{
}

int GenSQL::OutlineDrop(QTextStream& ts, const QString& owner, const QString& outline)
{
        ts << "drop outline " << outline << ";" << endl;
        return 0;
}

int GenSQL::OutlineRename(QTextStream& ts, const QString& owner, const QString& outline)
{
        ts << "alter outline " << outline << " rename to NEW_NAME;" << endl;
        return 0;
}

int GenSQL::OutlineCategoryChange(QTextStream& ts, const QString& owner, const QString& outline)
{
        ts << "alter outline " << outline << " change category to NEW_CATEGORY;" << endl;
        return 0;
}
int GenSQL::RBSOnline(QTextStream& ts, const QString& rbs)
{
        ts << "alter rollback segment " << rbs << " online;" << endl;
        return 0;
}

int GenSQL::RBSOffline(QTextStream& ts, const QString& rbs)
{
        ts << "alter rollback segment " << rbs << " offline;" << endl;
        return 0;
}

int GenSQL::RBSDrop(QTextStream& ts, const QString& rbs)
{
        ts << "drop rollback segment " << rbs << ";" << endl;
        return 0;
}
int GenSQL::RBSShrink(QTextStream& ts, const QString& rbs)
{
        ts << "alter rollback segment " << rbs << " shrink;" << endl;
        return 0;
}

int GenSQL::RBSCreate(QTextStream& ts, const QString& rbs)
{
        // Get default storage of rbs
        ObjParm op;
        op.addParm(rbs);
        QString sql=gSQLCache->getSQL("dba_rollback_segs_per_rbs");
        if (sql=="") return -3;
        GridDataDisplay* gdd_rbs = new GridDataDisplay(gConnObj,sql);
        if (gdd_rbs->retrieve(&op)==-1) return -2;
        QListViewItemIterator it_rbs(gdd_rbs);
        row *rbs_row=(row *)(it_rbs.current());
        long initial=(rbs_row->getCellbyName("initial_extent")).toLong()/1024;
        long next=(rbs_row->getCellbyName("next_extent")).toLong()/1024;
        QString max=rbs_row->getCellbyName("max_extents");
	if (max==QString::null) max="unlimited";
        QString min=rbs_row->getCellbyName("min_extents");
        QString opt=rbs_row->getCellbyName("optsize");
        //long opt=(rbs_row->getCellbyName("optsize")).toLong()/1024;
        QString tbs=rbs_row->getCellbyName("tablespace_name");
        delete gdd_rbs;
        gdd_rbs=0;

        ts << "create rollback segment " << rbs << endl;
        ts << "tablespace " << tbs << endl;
        ts << "storage (initial " << initial << " k" << endl;
        ts << "         next " << next << " k" << endl;
        ts << "         minextents " << min << endl;
        ts << "         maxextents " << max ;
	if (opt!=QString::null) 
	     ts << endl << "         optimal " << (opt.toLong()/1024) << " k" ;
	ts << ");" << endl;

        return 0;
}

int GenSQL::TBSAdddf(QTextStream& ts, const QString& tbs)
{
        // get first file of tablespace
        ObjParm op;
        op.addParm(tbs);
        QString sql=gSQLCache->getSQL("first_file");
        if (sql=="") return -3;
        GridDataDisplay* gdd= new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;
        QListViewItemIterator it(gdd);
        row* r=(row *)(it.current());
        QString file_name=r->getCellbyName("file_name");
        delete gdd;

        ts << "alter tablespace " << tbs << endl;
	ts << "add datafile '" << file_name << "'<!!!CHANGE NAME!!!> size <SIZE>M;" << endl;

        return 0;
}

int GenSQL::TBSOnline(QTextStream& ts, const QString& tbs)
{
        ts << "alter tablespace " << tbs << " online;" << endl;
        return 0;
}

int GenSQL::TBSOffline(QTextStream& ts, const QString& tbs)
{
        ts << "alter tablespace " << tbs << " offline;" << endl;
        return 0;
}

int GenSQL::TBSDrop(QTextStream& ts, const QString& tbs)
{
        ts << "drop tablespace " << tbs << ";" << endl;
        return 0;
}

int GenSQL::TBSCreate(QTextStream& ts, const QString& tbs)
{
        // get first file of tablespace
        ObjParm op;
        op.addParm(tbs);
        QString sql=gSQLCache->getSQL("first_file");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;
        QListViewItemIterator it(gdd);
        row *r=(row *)(it.current());
        QString file_name=r->getCellbyName("file_name");
        delete gdd;
        gdd=0;

        // Get size of tbs
        sql=gSQLCache->getSQL("tbs_size_per_tbs");
        if (sql=="") return -3;
        GridDataDisplay* gdd_df = new GridDataDisplay(gConnObj,sql);
        if (gdd_df->retrieve(&op)==-1) return -2;
        QListViewItemIterator it_sizedf(gdd_df);
        row *df=(row *)(it_sizedf.current());
        QString tbs_size=df->getCellbyName("tbs_size");
        delete gdd_df;
        gdd_df=0;

        // Get default storage of tbs
        sql=gSQLCache->getSQL("dba_tablespaces_per_tablespace");
        if (sql=="") return -3;
        GridDataDisplay* gdd_tbs = new GridDataDisplay(gConnObj,sql);
        if (gdd_tbs->retrieve(&op)==-1) return -2;
        QListViewItemIterator it_tbs(gdd_tbs);
        row *tbs_row=(row *)(it_tbs.current());
        long initial=(tbs_row->getCellbyName("initial_extent")).toLong()/1024;
        long next=(tbs_row->getCellbyName("next_extent")).toLong()/1024;
        QString max=tbs_row->getCellbyName("max_extents");
	if (max==QString::null) max="unlimited";
        QString min=tbs_row->getCellbyName("min_extents");
        QString pct=tbs_row->getCellbyName("pct_increase");
        QString contents=tbs_row->getCellbyName("contents");
        QString extent_management=tbs_row->getCellbyName("extent_management");
        QString allocation_type=tbs_row->getCellbyName("allocation_type");
        QString logging=tbs_row->getCellbyName("logging");
        delete gdd_tbs;
        gdd_tbs=0;

        ts << "create tablespace " << tbs << endl;
	ts << "datafile '" << file_name << "' size " << tbs_size << " m" << endl;
	if (logging!="") ts << logging << endl;
	if ((extent_management=="DICTIONARY")||(extent_management==""))
	{
		ts << "default storage (initial " << initial << " k" << endl;
		ts << "                 next " << next << " k" << endl;
		ts << "                 minextents " << min << endl;
		ts << "                 maxextents " << max << endl;
		ts << "                 pctincrease " << pct << ")" << endl;
		if (extent_management!="")
		     ts << "extent management " << extent_management << endl;
	}
	else
	{
		ts << "extent management " << extent_management << endl;
		ts << allocation_type;
		if (allocation_type=="UNIFORM")
			ts << " size " << next << " k" << endl;
	}
	ts << contents << ";" << endl;

	return 0;
}

int GenSQL::DFResize(QTextStream& ts, const QString& df)
{
        ObjParm op;
        op.addParm(df);
        QString sql=gSQLCache->getSQL("dba_data_files_per_df");
        if (sql=="") return -3;
        GridDataDisplay* gdd_df = new GridDataDisplay(gConnObj,sql);
        if (gdd_df->retrieve(&op)==-1) return -2;
        QListViewItemIterator it_sizedf(gdd_df);
        row *df_row=(row *)(it_sizedf.current());
        long df_size=df_row->getCellbyName("bytes").toLong()/1024/1024;
        delete gdd_df;
        gdd_df=0;

        ts << "alter database datafile '" << df << "' resize " << df_size << "M;" << endl;
        //ts << "alter database datafile '" << df << "' resize 5M;" << endl;
        return 0;
}

int GenSQL::DFOnline(QTextStream& ts, const QString& df)
{
        ts << "alter database datafile '" << df << "' online;" << endl;
        return 0;
}

int GenSQL::DFOffline(QTextStream& ts, const QString& df)
{
        ts << "alter database datafile '" << df << "' offline;" << endl;
        return 0;
}

int GenSQL::DFRename(QTextStream& ts, const QString& df)
{
        ts << "alter database rename file '" << df << "' to '" << df << ";" << endl;
        return 0;
}

int GenSQL::DFAutoextendOn(QTextStream& ts, const QString& df)
{
        ts << "alter database datafile '" << df << "' autoextend on next <INCR> maxsize <MAX>|UNLIMITED;" << endl;
        return 0;
}

int GenSQL::DFAutoextendOff(QTextStream& ts, const QString& df)
{
        ts << "alter database datafile '" << df << "' autoextend off;" << endl;
        return 0;
}

int GenSQL::RoleCreate(QTextStream& ts, const QString& role, const QString& pw)
{
	if (pw=="YES")
		ts << "create role " << role << " identified by " << role << ";" << endl;	
	else if (pw=="EXTERNAL")
		ts << "create role " << role << " identified externally;" << endl;	
	else
		ts << "create role " << role << " not identified;" << endl;	
		
	return 0;
}

int GenSQL::RoleDrop(QTextStream& ts, const QString& role)
{
        ts << "drop role " << role << ";" << endl;
        return 0;
}

int GenSQL::RoleGrant(QTextStream& ts, const QString& role)
{
	QString admin, grantee;
        ObjParm op;
        op.addParm(role);
        QString sql=gSQLCache->getSQL("dba_role_privs_per_role");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
                admin=((row *)it.current())->getCellbyName("admin_option");
                grantee=((row *)it.current())->getCellbyName("grantee");
                ts << "grant " << role << " to " << grantee ;
		if (admin=="YES") ts << " with admin option";
		ts << ";" << endl;
        }

        delete gdd;
	gdd=0;

        return 0;
}


int GenSQL::RoleTabPrivs(QTextStream& ts, const QString& role)
{
	return(UserTabPrivs(ts,role));
}

int GenSQL::RoleRolePrivs(QTextStream& ts, const QString& role)
{
	return(UserRolePrivs(ts,role));
}

int GenSQL::RoleSysPrivs(QTextStream& ts, const QString& role)
{
	return(UserSysPrivs(ts,role));
}

int GenSQL::RoleColPrivs(QTextStream& ts, const QString& role)
{
	return(UserColPrivs(ts,role));
}

int GenSQL::RolePrivs(QTextStream& ts, const QString& role)
{
	ts << "REM Table privs" << endl;
	int retval=RoleTabPrivs(ts,role);
	if (retval==0)
	{
		ts << "REM System privs" << endl;
		retval=RoleSysPrivs(ts,role);
		if (retval==0)
		{
			ts << "REM Role privs" << endl;
			retval=RoleRolePrivs(ts,role);
			if (retval==0)
			{
				ts << "REM Column privs" << endl;		
				retval=RoleColPrivs(ts,role);	
				if (retval==0) return 0;
				else return retval;
			}
			else return retval;
		}
		else return retval;
	}
	else return retval;
}

int GenSQL::CrdbsConfig(QTextStream& ts)
{
	long initial_extent, next_extent;
	QString tbs_name, min_extents;
	QString max_extents, pct_increase, tbs_size, tbs_type;
	int fs_num=1;
	row *r, *df;
	GridDataDisplay* gdd_df=0;

	// Get log mode
        QString sql=gSQLCache->getSQL("v_database");
        if (sql=="") return -3;
        GridDataDisplay* gdd_db = new GridDataDisplay(gConnObj,sql);
        if (gdd_db->retrieve()==-1) return -2;
        QListViewItemIterator it_db(gdd_db);
        r=(row *)(it_db.current());
	QString log_mode=r->getCellbyName("log_mode");
	QString dbname=r->getCellbyName("name");
	delete gdd_db;
	gdd_db=0;
	if (log_mode=="ARCHIVELOG") log_mode="Y";
	else log_mode="N";

	// Get char set
        ObjParm op_cs;
        op_cs.addParm("NLS_CHARACTERSET");
        sql=gSQLCache->getSQL("sys_props_per_prop");
        if (sql=="") return -3;
        GridDataDisplay* gdd_cs = new GridDataDisplay(gConnObj,sql);
        if (gdd_cs->retrieve(&op_cs)==-1) return -2;
        QListViewItemIterator it_cs(gdd_cs);
        r=(row *)(it_cs.current());
        QString charset=r->getCellbyName("value$");
        delete gdd_cs;
        gdd_cs=0;

        // Get NCHAR char set
        ObjParm op_ncs;
        op_ncs.addParm("NLS_NCHAR_CHARACTERSET");
        sql=gSQLCache->getSQL("sys_props_per_prop");
        if (sql=="") return -3;
        GridDataDisplay* gdd_ncs = new GridDataDisplay(gConnObj,sql);
        if (gdd_ncs->retrieve(&op_ncs)==-1) return -2;
        QListViewItemIterator it_ncs(gdd_ncs);
        r=(row *)(it_ncs.current());
        QString ncharset=r->getCellbyName("value$");
        delete gdd_ncs;
        gdd_ncs=0;

	// Get archive destination
        ObjParm op_ad;
        op_ad.addParm("log_archive_dest");
        sql=gSQLCache->getSQL("v_parameter_per_parameter");
        if (sql=="") return -3;
        GridDataDisplay* gdd_ad = new GridDataDisplay(gConnObj,sql);
        if (gdd_ad->retrieve(&op_ad)==-1) return -2;
        QListViewItemIterator it_ad(gdd_ad);
        r=(row *)(it_ad.current());
        QString archive_dest=r->getCellbyName("value");
	// Only need the path
	int dbname_pos=archive_dest.find(dbname);
	archive_dest=archive_dest.left(dbname_pos-1);
        delete gdd_cs;
        gdd_cs=0;

	// List of tablespaces
        sql=gSQLCache->getSQL("dba_tablespaces");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve()==-1) return -2;
        QListViewItemIterator it(gdd);

	int i=1;
        for (;it.current(); ++it)
        {
                r=(row *)(it.current());
		tbs_name=r->getCellbyName("tablespace_name");
		initial_extent=(r->getCellbyName("initial_extent")).toLong()/1024;
		next_extent=(r->getCellbyName("next_extent")).toLong()/1024;
		min_extents=r->getCellbyName("min_extents");
		max_extents=r->getCellbyName("max_extents");
		if (max_extents==QString::null) max_extents="unlimited";
		pct_increase=r->getCellbyName("pct_increase");
		
		// Get size of tbs
		ObjParm op_df;
        	op_df.addParm(tbs_name);
        	sql=gSQLCache->getSQL("tbs_size_per_tbs");
        	if (sql=="") return -3;
        	gdd_df = new GridDataDisplay(gConnObj,sql);
        	if (gdd_df->retrieve(&op_df)==-1) return -2;
        	QListViewItemIterator it_df(gdd_df);
        	df=(row *)(it_df.current());
        	tbs_size=df->getCellbyName("tbs_size");
		delete gdd_df;
		gdd_df=0;

		fs_num=(i % 2) + 1;

		if (tbs_name=="SYSTEM")
		{
			ts << dbname << " " << tbs_name << " " 
			   << tbs_size << " 2 " << log_mode 
			   << " 5 5 1 rbs " << charset << " " << ncharset << " "
			   << fs_num << " " << archive_dest << endl;
		}
		else 
		{
			if ((tbs_name=="RBS")||(tbs_name=="ROLLBACK"))
				tbs_type="ROLLBACK";
			else if ((tbs_name=="TEMP")||(tbs_name=="TEMPORARY"))
				tbs_type="TEMP";
			else if (tbs_name=="PATROL")
				tbs_type="PATROL";
			else if ((tbs_name=="AUDIT")||(tbs_name=="AUDITING"))
				tbs_type="AUDIT";
			else tbs_type="TABLE";


			ts << dbname << " " << tbs_type << " " << tbs_name.lower()
		   	   << " " << tbs_size << " " << initial_extent 
		   	   << " " << next_extent << " " << min_extents 
		   	   << " " << max_extents << " " << pct_increase
		   	   << " " << fs_num << endl;	
		}
		i++;	
        }

	delete gdd;
	gdd=0;
	return 0;
}

int GenSQL::DatabaseCreate(QTextStream& ts)
{
        long fsize;
        QString group_no,member,tbs_size;
        row *r, *mem, *df;
        GridDataDisplay* gdd_df=0;
        GridDataDisplay* gdd_log=0;

        // Get log mode and database name
        QString sql=gSQLCache->getSQL("v_database");
        if (sql=="") return -3;
        GridDataDisplay* gdd_db = new GridDataDisplay(gConnObj,sql);
        if (gdd_db->retrieve()==-1) return -2;
        QListViewItemIterator it_db(gdd_db);
        r=(row *)(it_db.current());
        QString log_mode=r->getCellbyName("log_mode");
        QString dbname=r->getCellbyName("name");
        delete gdd_db;
        gdd_db=0;


        // Get char set
        ObjParm op_cs;
        op_cs.addParm("NLS_CHARACTERSET");
        sql=gSQLCache->getSQL("sys_props_per_prop");
        if (sql=="") return -3;
        GridDataDisplay* gdd_cs = new GridDataDisplay(gConnObj,sql);
        if (gdd_cs->retrieve(&op_cs)==-1) return -2;
        QListViewItemIterator it_cs(gdd_cs);
        r=(row *)(it_cs.current());
        QString charset=r->getCellbyName("value$");
        delete gdd_cs;
        gdd_cs=0;


        // Get archive destination
        ObjParm op_ad;
        op_ad.addParm("log_archive_dest");
        sql=gSQLCache->getSQL("v_parameter_per_parameter");
        if (sql=="") return -3;
        GridDataDisplay* gdd_ad = new GridDataDisplay(gConnObj,sql);
        if (gdd_ad->retrieve(&op_ad)==-1) return -2;
        QListViewItemIterator it_ad(gdd_ad);
        r=(row *)(it_ad.current());
        QString archive_dest=r->getCellbyName("value");
        delete gdd_cs;
        gdd_cs=0;

	ts << "connect internal" << endl;
	ts << "startup nomount;" << endl;
	ts << "create database " << dbname << endl;
	ts << "\tcharacter set " << charset << endl;
	ts << "\tlogfile" << endl;

	// List of redo log groups
        sql=gSQLCache->getSQL("v_log");
        if (sql=="") return -3;
        GridDataDisplay* gdd_grp = new GridDataDisplay(gConnObj,sql);
        if (gdd_grp->retrieve()==-1) return -2;
        QListViewItemIterator it_grp(gdd_grp);

        for (;it_grp.current(); ++it_grp)
        {
                r=(row *)(it_grp.current());
                group_no=r->getCellbyName("group#");
                fsize=(r->getCellbyName("bytes")).toLong()/1024;

		if (group_no=="1")
			ts << "\t\tgroup " << group_no << " ('";
		else
			ts << "\t\t,group " << group_no << " ('";

		// Get list of member files
        	ObjParm op_log;
        	op_log.addParm(group_no);
	        sql=gSQLCache->getSQL("v_logfile_per_redolog_group");
        	if (sql=="") return -3;
        	gdd_log = new GridDataDisplay(gConnObj,sql);
        	if (gdd_log->retrieve(&op_log)==-1) return -2;
        	QListViewItemIterator it_log(gdd_log);
		
		// First member
                mem=(row *)(it_log.current());
                member=mem->getCellbyName("member");
		ts << member << "'";
		++it_log;

        	for (;it_log.current(); ++it_log)
        	{
                	mem=(row *)(it_log.current());
                	member=mem->getCellbyName("member");
			ts << endl << "\t\t       ,'" << member << "'" ;
		}

		ts << ") size " << fsize << " k" << endl;

		delete gdd_log;
		gdd_log=0;
	}
	delete gdd_grp;
	gdd_grp=0;

	// system tablespace
	// get first file of system tablespace
        ObjParm op_sys;
        op_sys.addParm("SYSTEM");
        sql=gSQLCache->getSQL("first_file");
        if (sql=="") return -3;
        GridDataDisplay* gdd_sys = new GridDataDisplay(gConnObj,sql);
        if (gdd_sys->retrieve(&op_sys)==-1) return -2;
        QListViewItemIterator it_sys(gdd_sys);
        r=(row *)(it_sys.current());
        QString system_file=r->getCellbyName("file_name");
        delete gdd_sys;
        gdd_cs=0;
        // Get size of system tbs
        sql=gSQLCache->getSQL("tbs_size_per_tbs");
        if (sql=="") return -3;
        gdd_df = new GridDataDisplay(gConnObj,sql);
        if (gdd_df->retrieve(&op_sys)==-1) return -2;
        QListViewItemIterator it_syssizedf(gdd_df);
        df=(row *)(it_syssizedf.current());
        tbs_size=df->getCellbyName("tbs_size");
        delete gdd_df;
        gdd_df=0;

	ts << "\tdatafile '" << system_file << "' size " << tbs_size << " m;" << endl;
	
        // Tablespace creation
        sql=gSQLCache->getSQL("dba_tablespaces");
        if (sql=="") return -3;
        GridDataDisplay* gdd_tbs = new GridDataDisplay(gConnObj,sql);
        if (gdd_tbs->retrieve()==-1) return -2;
        QListViewItemIterator it_tbs(gdd_tbs);

	QString tbs_name;
	int retval=0;
        for (;it_tbs.current(); ++it_tbs)
        {
                r=(row *)(it_tbs.current());
                tbs_name=r->getCellbyName("tablespace_name");
		if (tbs_name!="SYSTEM")
		{
			retval=TBSCreate(ts,tbs_name);
			if (retval!=0) return retval;
		}
        }

        delete gdd_tbs;
        gdd_tbs=0;

        // rbs creation
        sql=gSQLCache->getSQL("dba_rollback_segs");
        if (sql=="") return -3;
        GridDataDisplay* gdd_rbs = new GridDataDisplay(gConnObj,sql);
        if (gdd_rbs->retrieve()==-1) return -2;
        QListViewItemIterator it_rbs(gdd_rbs);

        QString rbs_name;
        retval=0;
        for (;it_rbs.current(); ++it_rbs)
        {
                r=(row *)(it_rbs.current());
                rbs_name=r->getCellbyName("segment_name");
                if (rbs_name!="SYSTEM")
                {
                        retval=RBSCreate(ts,rbs_name);
                        if (retval!=0) return retval;
                }
        }

        delete gdd_rbs;
        gdd_rbs=0;

        return 0;
}

int GenSQL::ViewCompile(QTextStream& ts, const QString& owner, const QString& view)
{
        ts << "alter view " << owner << "." << view << " compile;" << endl;
        return 0;
}

int GenSQL::ViewDrop(QTextStream& ts, const QString& owner, const QString& view)
{
        ts << "drop view " << owner << "." << view << ";" << endl;
        return 0;
}

int GenSQL::ViewCreate(QTextStream& ts, const QString& owner, const QString& view)
{
        ObjParm op;
        op.addParm(owner);
        op.addParm(view);
        QString sql=gSQLCache->getSQL("dba_views_per_view");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
        row* r=(row *)(it.current());

        QString text=r->getCellbyName("text");

        ts << "create or replace view " << owner << "." << view << "(" << endl;


	// Get view columns
	ObjParm op_cols;
        op_cols.addParm(owner);
        op_cols.addParm(view);
        sql=gSQLCache->getSQL("dba_tab_columns_per_table");
        if (sql=="") return -3;
        GridDataDisplay* gdd_cols = new GridDataDisplay(gConnObj,sql);
        if (gdd_cols->retrieve(&op_cols)==-1) return -2;

        QListViewItemIterator it_cols(gdd_cols);
        r=(row *)(it_cols.current());

        // First column (no leading comma)
        ts << "\t " << r->getCellbyName("COLUMN_NAME") << endl;
	++it_cols;


        // other columns
        for (;it_cols.current(); ++it_cols)
        {
                r=(row *)(it_cols.current());
                // write this column to file
                ts << "\t," << r->getCellbyName("COLUMN_NAME") << endl;
        }

        // closing column list
        ts << ")" << endl;
        delete gdd_cols;
        gdd_cols=0;
        ts << "as" << endl;

	// check if there is a view constraint
        ObjParm op_cons;
        op_cons.addParm(owner);
        op_cons.addParm(view);
        sql=gSQLCache->getSQL("dba_constraints_per_view");
        if (sql=="") return -3;
        GridDataDisplay* gdd_cons = new GridDataDisplay(gConnObj,sql);
	long num_rows=gdd_cons->retrieve(&op_cons);
        if (num_rows==-1) return -2;
	else if (num_rows==0) // no constraint 
	{
		ts << text << endl;
	}
	else if (num_rows > 0) // there is a constraint
	{
        	QListViewItemIterator it_cons(gdd_cons);
        	r=(row *)(it_cons.current());
        	QString constraint_name = r->getCellbyName("constraint_name");
        	ts << text << " constraint " << constraint_name << endl;
		delete gdd_cons;
		gdd_cons=0;
	}
        ts << "/" << endl;

        delete gdd;
        gdd=0;

        return 0;
}

int GenSQL::TriggerEnable(QTextStream& ts, const QString& owner, const QString& trigger)
{
	ts << "alter trigger " << owner << "." << trigger << " enable;" << endl;	
	return 0;
}

int GenSQL::TriggerDisable(QTextStream& ts, const QString& owner, const QString& trigger)
{
	ts << "alter trigger " << owner << "." << trigger << " disable;" << endl;	
	return 0;
}

int GenSQL::TriggerCompile(QTextStream& ts, const QString& owner, const QString& trigger)
{
	ts << "alter trigger " << owner << "." << trigger << " compile;" << endl;	
	return 0;
}

int GenSQL::TriggerDrop(QTextStream& ts, const QString& owner, const QString& trigger)
{
	ts << "drop trigger " << owner << "." << trigger << ";" << endl;	
	return 0;
}

int GenSQL::TriggerCreate(QTextStream& ts, const QString& owner, const QString& trigger)
{
        ObjParm op;
        op.addParm(owner);
        op.addParm(trigger);
        QString sql=gSQLCache->getSQL("dba_triggers_per_trigger");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
        row* r=(row *)(it.current());

        QString trigger_type=r->getCellbyName("trigger_type");
        //QString trigger_event=r->getCellbyName("trigger_event");
        QString table_owner=r->getCellbyName("table_owner");
        QString table_name=r->getCellbyName("table_name");
        QString referencing_names=r->getCellbyName("referencing_names");
        QString when_clause=r->getCellbyName("when_clause");
        QString trigger_body=r->getCellbyName("trigger_body");
        QString description=r->getCellbyName("description");

        ts << "create or replace trigger " << description << endl;
        if (when_clause!=QString::null) ts << "when (" << when_clause << ")" << endl;
        if (referencing_names!="REFERENCING NEW AS NEW OLD AS OLD") 
		ts << referencing_names << endl;
        ts << trigger_body << endl;
        ts << "/" << endl;

        delete gdd;
        gdd=0;

        return 0;
}

int GenSQL::SequenceDrop(QTextStream& ts, const QString& owner, const QString& sequence)
{
	ts << "drop sequence " << owner << "." << sequence << ";" << endl;	
	return 0;
}

int GenSQL::SequenceCreate(QTextStream& ts, const QString& owner, const QString& sequence)
{

        ObjParm op;
        op.addParm(owner);
        op.addParm(sequence);
        QString sql=gSQLCache->getSQL("dba_sequences_per_sequence");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
        row* r=(row *)(it.current());

        QString increment_by=r->getCellbyName("INCREMENT_BY");
        QString start_with=r->getCellbyName("LAST_NUMBER");
        QString max_value=r->getCellbyName("MAX_VALUE");
        QString min_value=r->getCellbyName("MIN_VALUE");
        QString cycle_flag=r->getCellbyName("CYCLE_FLAG");
        QString cache_size=r->getCellbyName("CACHE_SIZE");
        QString order_flag=r->getCellbyName("ORDER_FLAG");

	ts << "create sequence " << owner << "." << sequence << endl;
	ts << "increment by " << increment_by << endl;
	ts << "start with " << start_with.toInt()+1 << endl;
	if (max_value!=QString::null) ts << "maxvalue " << max_value << endl;
	if (min_value!=QString::null) ts << "minvalue " << min_value << endl;
	if (cycle_flag=="Y") ts << "cycle " << min_value << endl;
	if (cache_size!=QString::null) ts << "cache " << cache_size << endl;
	if (order_flag=="Y") ts << "order " << endl;
	ts << "/" << endl;
		

        delete gdd;
        gdd=0;

	return 0;
}


int GenSQL::IndexDrop(QTextStream& ts, const QString& owner, const QString& index)
{
	ts << "drop index " << owner << "." << index << ";" << endl;	
	return 0;
}

int GenSQL::IndexRebuild(QTextStream& ts, const QString& owner, const QString& index)
{
	ts << "alter index " << owner << "." << index << " rebuild;" << endl;	
	return 0;
}

int GenSQL::IndexCreate(QTextStream& ts, const QString& owner, const QString& index)
{

        ObjParm op;
        op.addParm(owner);
        op.addParm(index);
        QString sql=gSQLCache->getSQL("dba_indexes_per_index");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
	row* r=(row *)(it.current());

	QString uniqueness=r->getCellbyName("UNIQUENESS");
	QString table_type=r->getCellbyName("TABLE_TYPE");
	QString table_name=r->getCellbyName("TABLE_NAME");
	QString ini_trans=r->getCellbyName("INI_TRANS");
	QString max_trans=r->getCellbyName("MAX_TRANS");
//	QString tbs_trans=r->getCellbyName("TBS_TRANS");
	QString pct_free=r->getCellbyName("PCT_FREE");
	QString tablespace_name=r->getCellbyName("TABLESPACE_NAME");

	delete gdd;
	gdd=0;

	if (uniqueness=="UNIQUE")
        	ts << "create unique index " << owner << "." << index << endl;
	else
        	ts << "create index " << owner << "." << index << endl;

	if (table_type=="TABLE")
	{
		ts << "on " << owner << "." << table_name << " (" << endl;

		// Get index columns
        	ObjParm op_cols;
        	op_cols.addParm(owner);
        	op_cols.addParm(index);
        	sql=gSQLCache->getSQL("dba_ind_columns_per_index");
        	if (sql=="") return -3;
        	GridDataDisplay* gdd_cols = new GridDataDisplay(gConnObj,sql);
        	if (gdd_cols->retrieve(&op_cols)==-1) return -2;

        	QListViewItemIterator it_cols(gdd_cols);
		r=(row *)(it_cols.current());

        	// First column (no leading comma)
        	ts << "\t " << r->getCellbyName("COLUMN_NAME") << endl;
		++it_cols;


        	// other columns
        	for (;it_cols.current(); ++it_cols)
        	{
			r=(row *)(it_cols.current());
                	// write this column to file
                	ts << "\t," << r->getCellbyName("COLUMN_NAME") << endl;
        	}

        	// closing column list
        	ts << ")" << endl;
        	delete gdd_cols;
        	gdd_cols=0;
	}
	else if (table_type=="CLUSTER")
	{
		ts << "on " << owner << "." << table_name;
	}

        ts << "initrans " << ini_trans << endl;
        ts << "maxtrans " << max_trans << endl;
        ts << "tablespace " << tablespace_name << endl;

        // storage clause
        int retval=SegmentStorageClause(ts, owner, index, "INDEX");
        if (retval != 0) return retval;

        ts << "pctfree " << pct_free << endl;

        ts << "/" << endl;

        delete gdd;
        gdd=0;
        return 0;
}


int GenSQL::DBLinkDrop(QTextStream& ts, const QString& owner, const QString& link)
{
	if (owner=="PUBLIC")
        	ts << "drop public database link " << link << " ;" << endl;
	else
        	ts << "drop database link " << link << " ;" << endl;
        return 0;
}

int GenSQL::DBLinkCreate(QTextStream& ts, const QString& owner, const QString& link)
{
        ObjParm op;
        op.addParm(owner);
        op.addParm(link);
        QString sql=gSQLCache->getSQL("dba_db_links_per_db_link");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
        row * r=(row *)it.current();

        QString username=r->getCellbyName("username");
        QString password=r->getCellbyName("password");
        QString db_link=r->getCellbyName("db_link");
        QString host=r->getCellbyName("host");

	if (owner=="PUBLIC")
		ts << "create public ";
	else
		ts << "create ";

	ts << "database link " << db_link << " connect to " << username
	   << " identified by " << password ;
	if (host!="") ts << " using '" << host;
	ts << "';" << endl;

        delete gdd;
	gdd=0;

        return 0;
}


int GenSQL::SynonymDrop(QTextStream& ts, const QString& owner, const QString& synonym)
{
	ts << "drop synonym " << owner << "." << synonym << " ;" << endl;
	return 0;
}

int GenSQL::SynonymCreate(QTextStream& ts, const QString& owner, const QString& synonym)
{
        ObjParm op;
        op.addParm(owner);
        op.addParm(synonym);
        QString sql=gSQLCache->getSQL("dba_synonyms_per_synonym");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);
        row * r=(row *)it.current();

        QString table_owner=r->getCellbyName("table_owner");
        QString table_name=r->getCellbyName("table_name");
        QString db_link=r->getCellbyName("db_link");

	ts << "create synonym " << owner << "." << synonym << " for "
	   << table_owner << "." << table_name;

	if (db_link!=QString::null) ts << "@" << db_link ;

	ts << ";" << endl;


        delete gdd;
	gdd=0;

        return 0;
}

int GenSQL::FunctionCompile(QTextStream& ts, const QString& owner, const QString& function)
{
	ts << "alter function " << owner << "." << function << " compile;" << endl;
	return 0;
}

int GenSQL::FunctionDrop(QTextStream& ts, const QString& owner, const QString& function)
{
	ts << "drop function " << owner << "." << function << " ;" << endl;
	return 0;
}


int GenSQL::FunctionCreate(QTextStream& ts, const QString& owner, const QString& function)
{
	ts << "create or replace " << endl;
        ObjParm op;
        op.addParm(owner);
        op.addParm(function);
        QString sql=gSQLCache->getSQL("dba_source_per_function");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
                ts << ((row *)it.current())->getCellbyName("text") << endl;
	}

	delete gdd;
	gdd=0;

	ts << "/" << endl;

	return 0;
}

int GenSQL::ProcedureCompile(QTextStream& ts, const QString& owner, const QString& procedure)
{
        ts << "alter procedure " << owner << "." << procedure << " compile;" << endl;
        return 0;
}

int GenSQL::ProcedureDrop(QTextStream& ts, const QString& owner, const QString& procedure)
{
        ts << "drop procedure " << owner << "." << procedure << " ;" << endl;
        return 0;
}

int GenSQL::ProcedureCreate(QTextStream& ts, const QString& owner, const QString& procedure)
{
        ts << "create or replace " << endl;
        ObjParm op;
        op.addParm(owner);
        op.addParm(procedure);
        QString sql=gSQLCache->getSQL("dba_source_per_procedure");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
                ts << ((row *)it.current())->getCellbyName("text") << endl;
        }

        delete gdd;
	gdd=0;

        ts << "/" << endl;

        return 0;
}

int GenSQL::PackageCompile(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "alter package " << owner << "." << package << " compile;" << endl;
        return 0;
}

int GenSQL::PackageBodyCompile(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "alter package " << owner << "." << package << " compile body;" << endl;
        return 0;
}

int GenSQL::PackageDrop(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "drop package " << owner << "." << package << " ;" << endl;
        return 0;
}

int GenSQL::PackageBodyDrop(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "drop package body " << owner << "." << package << " ;" << endl;
        return 0;
}

int GenSQL::PackageCreate(QTextStream& ts, const QString& owner, const QString& package)
{
        int retval=PackageHeaderCreate(ts,owner,package);
        if (retval==0)
        {
        	retval=PackageBodyCreate(ts,owner,package);
                return retval;
        }
        else return retval;
}


int GenSQL::PackageHeaderCreate(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "create or replace " << endl;
        ObjParm op;
        op.addParm(owner);
        op.addParm(package);
        QString sql=gSQLCache->getSQL("dba_source_per_package_header");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
                ts << ((row *)it.current())->getCellbyName("text") << endl;
        }

        delete gdd;
	gdd=0;

        ts << "/" << endl;

        return 0;
}

int GenSQL::PackageBodyCreate(QTextStream& ts, const QString& owner, const QString& package)
{
        ts << "create or replace " << endl;
        ObjParm op;
        op.addParm(owner);
        op.addParm(package);
        QString sql=gSQLCache->getSQL("dba_source_per_package_body");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
                ts << ((row *)it.current())->getCellbyName("text") << endl;
        }

        delete gdd;
	gdd=0;

        ts << "/" << endl;

        return 0;
}

int GenSQL::ConstraintDrop(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename)
{
	ts << "alter table " << owner << "." << tablename << " drop constraint " << constraint << ";" << endl;
	return 0;
}

int GenSQL::ConstraintEnable(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename)
{
	ts << "alter table " << owner << "." << tablename << " enable constraint " << constraint << ";" << endl;
	return 0;
}

int GenSQL::ConstraintDisable(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename)
{
	ts << "alter table " << owner << "." << tablename << " disable constraint " << constraint << ";" << endl;
	return 0;
}

int GenSQL::ObjectCompileWithSchemaSwitch(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype)
{
	ts << "alter session set current_schema="+owner+";" << endl;
	ObjectCompile(ts,owner,objectname,objecttype);
	return 0;
}

int GenSQL::ObjectCompile(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype)
{
	if ((objecttype == "VIEW")
	|| (objecttype == "PROCEDURE")
	|| (objecttype == "TRIGGER")
	|| (objecttype == "INDEX")
	|| (objecttype == "INDEXTYPE")
	|| (objecttype == "OPERATOR")
	|| (objecttype == "PACKAGE")
	|| (objecttype == "PACKAGE BODY")
	|| (objecttype == "TYPE BODY")
	|| (objecttype == "FUNCTION")
	|| (objecttype == "TYPE"))

		if (objecttype=="PACKAGE BODY")
			ts << "alter package "+owner+"."+objectname+" compile body"+";" << endl;
		else if (objecttype=="TYPE BODY")
			ts << "alter type "+owner+"."+objectname+" compile body"+";" << endl;
		else
			ts << "alter " << objecttype << " " << owner << "." << objectname << " compile" << ";" << endl;
        return 0;
}

int GenSQL::ObjectDrop(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype)
{
	ts << "drop " << objecttype << " " << owner << "." << objectname << ";" << endl;
	return 0;
}

QString GenSQL::ConstructColumn(row * r)
{
	QString colname,nullable,datatype,datadefault,dataprec,datascale,datalength;
	QString sql;

                colname=r->getCellbyName("column_name");
                nullable=r->getCellbyName("nullable");
                datatype=r->getCellbyName("data_type");
                datadefault=r->getCellbyName("data_default");
                dataprec=r->getCellbyName("data_precision");
                datascale=r->getCellbyName("data_scale");
                datalength=r->getCellbyName("data_length");

                sql=colname+"             "+datatype;
                if (dataprec)
                {
                        sql=sql+"("+dataprec;
                        if (datascale) sql=sql+","+datascale;
                        sql=sql+")";
                }
                else if ((datatype!="LONG") && 
                         (datatype!="LONG RAW") &&
                         (datatype!="DATE") &&
                         (datatype!="ROWID"))
                {       
                        sql=sql+"("+datalength+")";
                }

                if (datadefault) sql=sql+"      "+"default "+datadefault;
                if (nullable=="N") sql=sql+"     "+"not null";

		return sql;
}

int GenSQL::TableDrop(QTextStream& ts, const QString& owner, const QString& tablename)
{
	ts << "drop table " << owner << "." << tablename << ";" << endl;	
	return 0;
}

int GenSQL::TableTruncate(QTextStream& ts, const QString& owner, const QString& tablename)
{
	ts << "truncate table " << owner << "." << tablename << ";" << endl;	
	return 0;
}

int GenSQL::TableDeallocate(QTextStream& ts, const QString& owner, const QString& tablename)
{
	ts << "alter table " << owner << "." << tablename << " deallocate unused;" << endl;	
	return 0;
}
int GenSQL::TableCreateWithConstraints(QTextStream& ts, const QString& owner, 
				       const QString& tablename)
{
	// basic table
	int retval=TableCreate(ts,owner,tablename);
	if (retval!=0) return retval;

	// add constraints
        ObjParm op;
        op.addParm(owner);
        op.addParm(tablename);
        QString sql=gSQLCache->getSQL("dba_constraints_per_table_p_r_c");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        for (;it.current(); ++it)
        {
		// filter out NOT NULL constraints
		if ((((row *)it.current())->getCellbyName("search_condition")).right(11)!="IS NOT NULL") // the last 11 chars because the 
		      // beginning contains variable data
		// this cannot be filtered out using sql because search_condition
  		// is a long column
		{
                	// create constraint
                	retval=ConstraintCreate(ts,((row *)it.current())->getCellbyName("constraint_name"),owner,tablename);
			// Check the outcome of the constraint creation
			// If it failed bail out.
			if (retval!=0)
			{
				delete gdd;
				gdd=0;
				return retval;
			}
		}
        }
	delete gdd;
	gdd=0;
	return 0;
}
                
int GenSQL::TableCreate(QTextStream& ts, const QString& owner, const QString& tablename)
{
        ts << "create table " << owner << "." << tablename << endl;
        ts << "(" << endl;

        ObjParm op;
        op.addParm(owner);
        op.addParm(tablename);
        QString sql=gSQLCache->getSQL("dba_tab_columns_per_table");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
	if (gdd->retrieve(&op)==-1) return -2;

	QListViewItemIterator it(gdd);

	// First column (no leading comma)
	ts << "	" << ConstructColumn(((row *)it.current())) << endl;;
	it++;

	// other columns
        for (;it.current(); ++it)
        {
		// write this column to file
		ts << "	," << ConstructColumn(((row *)it.current())) << endl;;
        }

	// closing column list
        ts << ")" << endl;
	delete gdd;
	gdd=0;

	// Get storage parameters
        sql=gSQLCache->getSQL("dba_tables_params");
        if (sql=="") return -3;
        gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;
	QListViewItemIterator it2(gdd);

	row* r=(row *)(it2.current());

	// cluster clause
	QString cluster=r->getCellbyName("cluster_name");

	if (cluster==QString::null)
	{
		ts << "pctfree " << r->getCellbyName("pct_free") << endl;
		ts << "pctused " << r->getCellbyName("pct_used") << endl;
		ts << "initrans " << r->getCellbyName("ini_trans") << endl;
		ts << "maxtrans " << r->getCellbyName("max_trans") << endl;
		ts << "tablespace " << r->getCellbyName("tablespace_name") << endl;

		// storage clause
		int retval=SegmentStorageClause(ts, owner, tablename, "TABLE");
		if (retval != 0) return retval;
	}
	else ts << "cluster " << cluster << endl;

	// parallel clause
	QString parallel=r->getCellbyName("degree");
	ts << "parallel (degree " << parallel << ")" << endl;

	ts << "/" << endl;

	delete gdd;	
	gdd=0;
        return 0;
}

int GenSQL::UserBecome(QTextStream& ts, const QString& username)
{
	ts << "column c1 new_value pw" << endl;
	ts << "select password c1 from dba_users" << endl;
	ts << "where username='" << username << "'" << endl;
	ts << "/" << endl;
	ts << "alter user " << username << " identified by " << username << endl;
	ts << "/" << endl;
	ts << "connect " << username << "/" << username;
	if (gTNSAlias!="") ts << "@" << gTNSAlias ;
	ts << endl;

	ts << "alter user " << username << " identified by values '&pw'" << endl;
	ts << "/" << endl;
	return 0;
}
	
int GenSQL::UserCreate(QTextStream& ts, const QString& username)
{
        ObjParm op;
        op.addParm(username);
        QString sql=gSQLCache->getSQL("dba_users_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r=(row *)it.current();
	

	ts << "create user " << username << endl;
	ts << "identified by values '" << r->getCellbyName("password") << "'" << endl;
	ts << "default tablespace " << r->getCellbyName("default_tablespace") << endl;	
	ts << "temporary tablespace " << r->getCellbyName("temporary_tablespace") << endl;	

        sql=gSQLCache->getSQL("dba_ts_quotas_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd_quota = new GridDataDisplay(gConnObj,sql);
        if (gdd_quota->retrieve(&op)==-1) return -2;

        QListViewItemIterator it_quota(gdd_quota);

        for (;it_quota.current(); ++it_quota)
        {
		r=(row *)it_quota.current();
		QString quota = r->getCellbyName("max_bytes");
		if (quota == "-1") quota="unlimited";
		ts << "quota " << quota << " on " << r->getCellbyName("tablespace_name") << endl;
	}

	ts << "/" << endl;

	delete gdd;
	gdd=0;
	delete gdd_quota;
	gdd_quota=0;

	return 0;
}

int GenSQL::UserUnlock(QTextStream& ts, const QString& username)
{
        ts << "alter user " << username << " account unlock;" << endl;
        return 0;
}

int GenSQL::UserLock(QTextStream& ts, const QString& username)
{
        ts << "alter user " << username << " account lock;" << endl;
        return 0;
}

int GenSQL::UserDrop(QTextStream& ts, const QString& username)
{
        ts << "drop user " << username << " cascade;" << endl;
        return 0;
}


int GenSQL::UserTabPrivs(QTextStream& ts, const QString& username)
{
	QString sql, owner, table_name, grantor, privilege,grantable;
	QString object_type;

        ObjParm op;
        op.addParm(username);
        sql=gSQLCache->getSQL("dba_tab_privs_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        //GridDataDisplay* gdd = new GridDataDisplay(gConnObj,gSQLCache->getSQL("dba_tab_privs_per_user"));
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

	row * r;
        for (;it.current(); ++it)
        {
		r=(row *)it.current();
		owner=r->getCellbyName("owner");
		table_name=r->getCellbyName("table_name");
		grantor=r->getCellbyName("grantor");
		privilege=r->getCellbyName("privilege");
		grantable=r->getCellbyName("grantable");
		object_type=r->getCellbyName("object_type");

		if (object_type=="DIRECTORY")
			sql="grant "+privilege+" on directory "+grantor+"."+table_name+" to "+username;
		else
			sql="grant "+privilege+" on "+grantor+"."+table_name+" to "+username;
		if (grantable=="YES") sql=sql+" with grant option;";
		else sql=sql+";";
		ts << sql << endl;
	}

	delete gdd;
	gdd=0;
	return 0;
}

int GenSQL::UserRolePrivs(QTextStream& ts, const QString& username)
{
        QString sql, owner, granted_role, admin_option, default_role;

        ObjParm op;
        op.addParm(username);
	sql=gSQLCache->getSQL("dba_role_privs_per_user");
	if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        //GridDataDisplay* gdd = new GridDataDisplay(gConnObj,gSQLCache->getSQL("dba_role_privs_per_user"));
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r;
        for (;it.current(); ++it)
        {
                r=(row *)it.current();
                granted_role=r->getCellbyName("granted_role");
                admin_option=r->getCellbyName("admin_option");
                default_role=r->getCellbyName("default_role");

                sql="grant "+granted_role+" to "+username;
                if (admin_option=="YES") sql=sql+" with admin option;";
                else sql=sql+";";
                ts << sql << endl;
        }

        delete gdd;
	gdd=0;
        return 0;
}

int GenSQL::UserSysPrivs(QTextStream& ts, const QString& username)
{
        QString sql, privilege, admin_option;

        ObjParm op;
        op.addParm(username);
	sql=gSQLCache->getSQL("dba_sys_privs_per_user");
	if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r;
        for (;it.current(); ++it)
        {
                r=(row *)it.current();
                privilege=r->getCellbyName("privilege");
                admin_option=r->getCellbyName("admin_option");

                sql="grant "+privilege+" to "+username;
                if (admin_option=="YES") sql=sql+" with grant option;";
                else sql=sql+";";
                ts << sql << endl;
        }

        delete gdd;
	gdd=0;
        return 0;
}

int GenSQL::UserColPrivs(QTextStream& ts, const QString& username)
{
        QString sql, owner, table_name, column_name, grantor, privilege, grantable;

        ObjParm op;
        op.addParm(username);
        sql=gSQLCache->getSQL("dba_col_privs_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r;
        for (;it.current(); ++it)
        {
                r=(row *)it.current();
                owner=r->getCellbyName("owner");
                table_name=r->getCellbyName("table_name");
                column_name=r->getCellbyName("column_name");
                grantor=r->getCellbyName("grantor");
                privilege=r->getCellbyName("privilege");
                grantable=r->getCellbyName("grantable");

                sql="grant "+privilege+" ( "+ column_name + " ) "+" on "+grantor+"."+table_name+" to "+username;
                if (grantable=="YES") sql=sql+" with grant option;";
                else sql=sql+";";
                ts << sql << endl;
        }

        delete gdd;
	gdd=0;
        return 0;
}

int GenSQL::UserPrivs(QTextStream& ts, const QString& username)
{
	ts << "REM Table privs" << endl;
	int retval=UserTabPrivs(ts,username);
	if (retval==0)
	{
		ts << "REM System privs" << endl;
		retval=UserSysPrivs(ts,username);
		if (retval==0)
		{
			ts << "REM Role privs" << endl;
			retval=UserRolePrivs(ts,username);
			if (retval==0)
			{
				ts << "REM Column privs" << endl;		
				retval=UserColPrivs(ts,username);	
				if (retval==0) return 0;
				else return retval;
			}
			else return retval;
		}
		else return retval;
	}
	else return retval;
}

int GenSQL::UserTabGrants(QTextStream& ts, const QString& username)
{
        QString sql, owner, table_name, grantee, privilege, grantable;

        ObjParm op;
        op.addParm(username);
        sql=gSQLCache->getSQL("table_grants_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r;
        for (;it.current(); ++it)
        {
                r=(row *)it.current();
                owner=r->getCellbyName("owner");
                table_name=r->getCellbyName("table_name");
                grantee=r->getCellbyName("grantee");
                privilege=r->getCellbyName("privilege");
                grantable=r->getCellbyName("grantable");

                sql="grant "+privilege+" on "+username+"."+table_name+" to "+grantee;
                if (grantable=="YES") sql=sql+" with grant option;";
                else sql=sql+";";
                ts << sql << endl;
        }

        delete gdd;
	gdd=0;
        return 0;
}

int GenSQL::UserColGrants(QTextStream& ts, const QString& username)
{
        QString sql, owner, table_name, column_name, grantee, privilege, grantable;

        ObjParm op;
        op.addParm(username);
        sql=gSQLCache->getSQL("col_grants_per_user");
        if (sql=="") return -3;
        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r;
        for (;it.current(); ++it)
        {
                r=(row *)it.current();
                owner=r->getCellbyName("owner");
                table_name=r->getCellbyName("table_name");
                column_name=r->getCellbyName("column_name");
                grantee=r->getCellbyName("grantee");
                privilege=r->getCellbyName("privilege");
                grantable=r->getCellbyName("grantable");

                sql="grant "+privilege+" ( "+ column_name + " ) "+" on "+username+"."+table_name+" to "+grantee;
                if (grantable=="YES") sql=sql+" with grant option;";
                else sql=sql+";";
                ts << sql << endl;
        }

        delete gdd;
	gdd=0;
        return 0;
}

int GenSQL::UserGrants(QTextStream& ts, const QString& username)
{
        ts << "REM Table grants" << endl;
        int retval=UserTabGrants(ts,username);
        if (retval==0)
        {
                ts << "REM Column grants" << endl;
                retval=UserColGrants(ts,username);
                if (retval==0) return 0;
                else return retval;
        }
        else return retval;
}

int GenSQL::SessionKill(QTextStream& ts, const QString& sid, const QString& serial)
{
	ts << "alter system kill session '" << sid << "," << serial << "';" << endl;	
	return 0;
}

int GenSQL::SessionTrace(QTextStream& ts, const QString& sid, const QString& serial)
{
	ts << "execute dbms_system.set_sql_trace_in_session(" << sid << "," << serial << ",TRUE);" << endl;	
	return 0;
}

int GenSQL::SegmentStorageClause(QTextStream& ts, const QString& owner, const QString& segmentname, const QString& segmenttype)
{
        QString sql;

        ObjParm op;
        op.addParm(owner);
        op.addParm(segmentname);
        op.addParm(segmenttype);
        sql=gSQLCache->getSQL("dba_segments_storage");
        if (sql=="") return -3;

        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;

        QListViewItemIterator it(gdd);

        row * r=(row *)it.current();

	ts << "storage (" << endl;
	ts << "\tinitial " << r->getCellbyName("initial_extent") << endl;
	ts << "\tnext " << r->getCellbyName("next_extent") << endl;
	ts << "\tminextents " << r->getCellbyName("min_extents") << endl;
	ts << "\tmaxextents " << r->getCellbyName("max_extents") << endl;
	ts << "\tpctincrease " << r->getCellbyName("pct_increase") << endl;
	ts << "\tfreelists " << r->getCellbyName("freelists") << endl;
	ts << "\tfreelist groups " << r->getCellbyName("freelist_groups") << endl;
	ts << ")" << endl;

	return 0;
}

int GenSQL::ConstraintCreate(QTextStream& ts, const QString& constraint, 
			     const QString& owner, const QString& tablename)
{
	ts << "alter table " << owner << "." << tablename << " add (" << endl;
	ts << "\tconstraint " << constraint << endl;


        ObjParm op;
        op.addParm(owner);
        op.addParm(constraint);
        QString sql=gSQLCache->getSQL("dba_constraints_per_constraint");
        if (sql=="") return -3;

        GridDataDisplay* gdd = new GridDataDisplay(gConnObj,sql);
        if (gdd->retrieve(&op)==-1) return -2;
        QListViewItemIterator it(gdd);
        row * r=(row *)it.current();

	// Check constraint
	QString constraint_type=r->getCellbyName("constraint_type");
	if (constraint_type=="C")
		ts << "\tcheck (" << r->getCellbyName("search_condition") << ")" << endl;
	else // unique, primary, foreign
	{
        	sql=gSQLCache->getSQL("dba_cons_columns_per_constraint");
        	if (sql=="") return -3;

        	GridDataDisplay* cols = new GridDataDisplay(gConnObj,sql);
        	if (cols->retrieve(&op)==-1) return -2;
        	QListViewItemIterator it_cols(cols);
        	row * c=(row *)it_cols.current();

		if (constraint_type=="U")
			ts << "\tunique (" << endl;
		else if (constraint_type=="P")
			ts << "\tprimary key (" << endl;
		else if (constraint_type=="R")
			ts << "\tforeign key (" << endl;

		// first column
		ts << "\t\t" << c->getCellbyName("column_name") << endl;
		it_cols++;
		// other columns
        	for (;it_cols.current(); ++it_cols)
        	{
			c=(row *)it_cols.current();
			ts << "\t\t," << c->getCellbyName("column_name") << endl;
		}
		ts << "\t\t\t)" << endl;
			
		// special for unique and primary
		if ((constraint_type=="U")||(constraint_type=="P"))
		{
			ts << "using index" << endl;
			
			// get index 
                	sql=gSQLCache->getSQL("dba_indexes_per_index");
                	if (sql=="") return -3;

                	GridDataDisplay* idx = new GridDataDisplay(gConnObj,sql);
                	if (idx->retrieve(&op)==-1) return -2;
                	QListViewItemIterator it_idx(idx);
                	row * i=(row *)it_idx.current();

			// pct_free 
			ts << "pctfree " << i->getCellbyName("pct_free") << endl;
			ts << "initrans " << i->getCellbyName("ini_trans") << endl;
			ts << "maxtrans " << i->getCellbyName("max_trans") << endl;
			ts << "tablespace " << i->getCellbyName("tablespace_name") << endl;

			
			// storage clause
			int retval=SegmentStorageClause(ts, owner, constraint, "INDEX");
			if (retval != 0) return retval;

			delete idx;
			idx=0;
		}

		// special for foreign keys
                if (constraint_type=="R")
                {
        		ObjParm op_rcons;
        		op_rcons.addParm(r->getCellbyName("r_owner"));
        		op_rcons.addParm(r->getCellbyName("r_constraint_name"));
                        // get referenced table
                        sql=gSQLCache->getSQL("dba_cons_columns_per_constraint");
                        if (sql=="") return -3;

                        GridDataDisplay* rcons = new GridDataDisplay(gConnObj,sql);
                        if (rcons->retrieve(&op_rcons)==-1) return -2;
                        QListViewItemIterator it_rcons(rcons);
                        row * rc=(row *)it_rcons.current();

                        ts << "\treferences " << r->getCellbyName("r_owner") << "." << rc->getCellbyName("table_name") << " (" << endl;

                        // get 
                        sql=gSQLCache->getSQL("dba_cons_columns_per_constraint");
                        if (sql=="") return -3;

                        GridDataDisplay* rcols = new GridDataDisplay(gConnObj,sql);
                        if (rcols->retrieve(&op_rcons)==-1) return -2;
                        QListViewItemIterator it_rcols(rcols);
                        row * rcol=(row *)it_rcols.current();

                	// first column
                	ts << "\t\t" << rcol->getCellbyName("column_name") << endl;
                	it_rcols++;
                	// other columns
                	for (;it_rcols.current(); ++it_rcols)
                	{
                		ts << "\t\t," << rcol->getCellbyName("column_name") << endl;
                	}
			ts << "\t\t)" << endl;

			delete rcols;
			rcols=0;
                        delete rcons;
			rcons=0;

			if (r->getCellbyName("delete_rule")=="CASCADE")
				ts << "\ton delete cascade" << endl;
			
                }


		delete cols;
		cols=0;
	}
	delete gdd;
	gdd=0;

	// last parenthese
	ts << ")" << endl;

	ts << "/" << endl;
	return 0;
}
