/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#ifndef GENSQL_H
#define GENSQL_H

class QTextStream;
class QString;
class row;

class SQLCache;
extern SQLCache * gSQLCache;
class qt_oraconnect;
extern qt_oraconnect * gConnObj;

class GenSQL
{
public:
	GenSQL();
	virtual ~GenSQL();

	static int CrdbsConfig(QTextStream& ts);
	static int DatabaseCreate(QTextStream& ts);

	static int RoleCreate(QTextStream& ts, const QString& role, const QString& pw);

	static int OutlineDrop(QTextStream& ts, const QString& owner, const QString& outline);
	static int OutlineRename(QTextStream& ts, const QString& owner, const QString& outline);
	static int OutlineCategoryChange(QTextStream& ts, const QString& owner, const QString& outline);

	static int RBSCreate(QTextStream& ts, const QString& rbs);
	static int RBSDrop(QTextStream& ts, const QString& rbs);
	static int RBSShrink(QTextStream& ts, const QString& rbs);
	static int RBSOffline(QTextStream& ts, const QString& rbs);
	static int RBSOnline(QTextStream& ts, const QString& rbs);

	static int TBSCreate(QTextStream& ts, const QString& tbs);
	static int TBSDrop(QTextStream& ts, const QString& tbs);
	static int TBSOnline(QTextStream& ts, const QString& tbs);
	static int TBSOffline(QTextStream& ts, const QString& tbs);
	static int TBSAdddf(QTextStream& ts, const QString& tbs);

	static int DFResize(QTextStream& ts, const QString& df);
	static int DFAutoextendOn(QTextStream& ts, const QString& df);
	static int DFAutoextendOff(QTextStream& ts, const QString& df);
	static int DFOnline(QTextStream& ts, const QString& df);
	static int DFOffline(QTextStream& ts, const QString& df);
	static int DFRename(QTextStream& ts, const QString& df);

	static int RoleDrop(QTextStream& ts, const QString& role);
	static int RoleGrant(QTextStream& ts, const QString& role);
	static int RolePrivs(QTextStream& ts, const QString& role);
	static int RoleTabPrivs(QTextStream& ts, const QString& role);
	static int RoleRolePrivs(QTextStream& ts, const QString& role);
	static int RoleSysPrivs(QTextStream& ts, const QString& role);
	static int RoleColPrivs(QTextStream& ts, const QString& role);

	static int ViewCreate(QTextStream& ts, const QString& owner, const QString& view);
	static int ViewDrop(QTextStream& ts, const QString& owner, const QString& view);
	static int ViewCompile(QTextStream& ts, const QString& owner, const QString& view);

	static int TriggerCreate(QTextStream& ts, const QString& owner, const QString& trigger);
	static int TriggerDrop(QTextStream& ts, const QString& owner, const QString& trigger);
	static int TriggerCompile(QTextStream& ts, const QString& owner, const QString& trigger);
	static int TriggerEnable(QTextStream& ts, const QString& owner, const QString& trigger);
	static int TriggerDisable(QTextStream& ts, const QString& owner, const QString& trigger);

	static int SequenceCreate(QTextStream& ts, const QString& owner, const QString& sequence);
	static int SequenceDrop(QTextStream& ts, const QString& owner, const QString& sequence);

	static int IndexCreate(QTextStream& ts, const QString& owner, const QString& index);
	static int IndexDrop(QTextStream& ts, const QString& owner, const QString& index);
	static int IndexRebuild(QTextStream& ts, const QString& owner, const QString& index);

	static int DBLinkDrop(QTextStream& ts, const QString& owner, const QString& link);
	static int DBLinkCreate(QTextStream& ts, const QString& owner, const QString& link);
	static int SynonymDrop(QTextStream& ts, const QString& owner, const QString& synonym);
	static int SynonymCreate(QTextStream& ts, const QString& owner, const QString& synonym);

	static int FunctionCompile(QTextStream& ts, const QString& owner, const QString& function);
	static int FunctionDrop(QTextStream& ts, const QString& owner, const QString& function);
	static int FunctionCreate(QTextStream& ts, const QString& owner, const QString& function);

	static int ProcedureCompile(QTextStream& ts, const QString& owner, const QString& procedure);
	static int ProcedureDrop(QTextStream& ts, const QString& owner, const QString& procedure);
	static int ProcedureCreate(QTextStream& ts, const QString& owner, const QString& procedure);

	static int PackageCompile(QTextStream& ts, const QString& owner, const QString& package);
	static int PackageBodyCompile(QTextStream& ts, const QString& owner, const QString& package);
	static int PackageDrop(QTextStream& ts, const QString& owner, const QString& procedure);
	static int PackageBodyDrop(QTextStream& ts, const QString& owner, const QString& procedure);
	static int PackageCreate(QTextStream& ts, const QString& owner, const QString& package);
	static int PackageBodyCreate(QTextStream& ts, const QString& owner, const QString& package);
	static int PackageHeaderCreate(QTextStream& ts, const QString& owner, const QString& package);

	static int ConstraintEnable(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename);
	static int ConstraintDrop(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename);
	static int ConstraintDisable(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename);
	static int ConstraintCreate(QTextStream& ts, const QString& constraint, const QString& owner, const QString& tablename);

	static int TableCreate(QTextStream& ts, const QString& owner, 
			       const QString& tablename);
	static int TableCreateWithConstraints(QTextStream& ts, 
					      const QString& owner, 
					      const QString& tablename);
	static int TableDrop(QTextStream& ts, const QString& owner, const QString& tablename);
	static int TableTruncate(QTextStream& ts, const QString& owner, const QString& tablename);
	static int TableDeallocate(QTextStream& ts, const QString& owner, const QString& tablename);

	static int SegmentStorageClause(QTextStream& ts, const QString& owner, const QString& segmentname, const QString& segmenttype);

	static int ObjectCompileWithSchemaSwitch(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype);
	static int ObjectCompile(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype);
	static int ObjectDrop(QTextStream& ts, const QString& owner, const QString& objectname, const QString& objecttype);

	static int UserCreate(QTextStream& ts, const QString& username);
	static int UserDrop(QTextStream& ts, const QString& username);
	static int UserBecome(QTextStream& ts, const QString& username);
	static int UserLock(QTextStream& ts, const QString& username);
	static int UserUnlock(QTextStream& ts, const QString& username);
	static int UserTabPrivs(QTextStream& ts, const QString& username);
	static int UserRolePrivs(QTextStream& ts, const QString& username);
	static int UserSysPrivs(QTextStream& ts, const QString& username);
	static int UserColPrivs(QTextStream& ts, const QString& username);
	static int UserPrivs(QTextStream& ts, const QString& username);
	static int UserTabGrants(QTextStream& ts, const QString& username);
	static int UserColGrants(QTextStream& ts, const QString& username);
	static int UserGrants(QTextStream& ts, const QString& username);

	static int SessionKill(QTextStream& ts, const QString& sid, const QString& serial);
	static int SessionTrace(QTextStream& ts, const QString& sid, const QString& serial);

private:
	static QString ConstructColumn(row * r);
};

#endif
