/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "listview.h" 
#include "row.h"
#include "general.h"
#include <iostream.h>
#include <qpaintdevicemetrics.h>
#include <qheader.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
// For qt3
#include <qpainter.h>

//ListView::ListView(QWidget * parent, const char * name) 
ListView::ListView(QWidget * parent, const QString& name) 
	:QListView(parent,name.local8Bit())
{
	setSorting(-1);
	// Convert these signal to their counterparts that have less or other
	// arguments
	connect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(emitRightButtonPressed(QListViewItem *, const QPoint &, int)));
	connect(this, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(emitReturnPressed(QListViewItem *)));
	connect(this, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(emitDoubleClicked(QListViewItem *)));
}

void ListView::emitRightButtonPressed(QListViewItem * r, const QPoint & point, int col)
{
	emit rightButtonPressed(point);
}

void ListView::emitDoubleClicked(QListViewItem * r)
{
	emit doubleClicked((row *) r);
}

void ListView::emitReturnPressed(QListViewItem * r)
{
	emit returnPressed((row *) r);
}

void ListView::showEvent(QShowEvent * e)
{
	emit shown();
}

void ListView::hideEvent(QHideEvent * e)
{
	emit hidden();
}

void ListView::printData(const QString& OutputFilename, const QString& docName, const QPrinter::Orientation& orient)
{
        const int Margin = 10;
        QPrinter *printer = new QPrinter();
	printer->setOutputFileName(OutputFilename);
	printer->setDocName(docName);
	printer->setOrientation(orient);
        if ( printer->setup(this) )
        {
		// Check if there is no column that is too large to be printed 
		// on a single page
		int s; // section
		int col_index=0;
                QPaintDeviceMetrics metrics( printer ); // need width/height
                                                        // of printer surface
                for (;col_index<columns();col_index++)
		{
                        s=header()->mapToSection(col_index);
			if (Margin+header()->sectionSize(s)>metrics.width())
			{
				QMessageBox::critical(this,"Printing problem",
				  	"The column '"+header()->label(s)+"' is too large to be printed on a single page.\nPlease reduce the size of the column or change the paper orientation and retry.",
					QMessageBox::Ok | QMessageBox::Default,0,0);
				delete printer;
				printer=0;
				return;
			}
		}
                QPainter p;
                p.begin( printer );                     // paint on printer
                int yPos        = 0;                    // y position for each line
                QFontMetrics fm = p.fontMetrics();
                //QPaintDeviceMetrics metrics( printer ); // need width/height
                                                        // of printer surface
                // print col heading of first page
                int total_width=0;
		int first_col=0; // first col for current page
		int last_col=0;  // last col for current page
		//int s; // section
		//int col_index=0;
		col_index=0;
                for (;col_index<columns();col_index++)
                {
                        s=header()->mapToSection(col_index);
                        if (Margin+total_width+(header()->sectionSize(s))>metrics.width())
			{
				break;
			}
                        p.drawRect( Margin+total_width, Margin + yPos, (header()->sectionSize(s)), fm.lineSpacing()+fm.underlinePos());
                        p.drawText( Margin+total_width, Margin + yPos,(header()->sectionSize(s)),
                                    fm.lineSpacing(), ExpandTabs , header()->label(s));
                        total_width=total_width+(header()->sectionSize(s));
                }
		last_col=col_index-1;
                yPos = yPos + fm.lineSpacing()+ fm.underlinePos();

                QListViewItemIterator it( this );
                QListViewItemIterator it_page_start=it;
                // iterate through all items of the listview
		int rows_on_page=0;
		// Iterate as long as there are rows or colums that have not been printed
                for ( ; ((it.current())||(last_col<columns()-1)); ++it )
                {
                        if ((Margin + yPos > (metrics.height()) - Margin)||((!it.current())&&(last_col<columns()-1)))
                        {
                                printer->newPage();             // no more room on this page
                                yPos = 0;                       // back to top of page
                                // print col heading
                                int total_width=0;
				if (last_col==columns()-1)
				{
					first_col=0;
					it_page_start=it;
					//pageNo++;
				}
				else // not yet all cols have been printed for the current rows
				{
					first_col=last_col+1;
					// jump back to the top of the current page
					//it-=(rows_on_page-1);
					it=it_page_start;
				}

				int col_index;
                		for (col_index=first_col;col_index<columns();col_index++)
                                {
                        		s=header()->mapToSection(col_index);
                                        if (Margin+total_width+(header()->sectionSize(s))>metrics.width()) 
					{
						break;
					}
                                        p.drawRect( Margin+total_width, Margin + yPos, (header()->sectionSize(s)),
                                                    fm.lineSpacing()+fm.underlinePos());
                                        p.drawText( Margin+total_width, Margin + yPos, (header()->sectionSize(s)),
                                                    fm.lineSpacing(), ExpandTabs , header()->label(s));
                                        total_width=total_width+(header()->sectionSize(s));
                                }
				last_col=col_index-1;
                                yPos = yPos + fm.lineSpacing()+ fm.underlinePos();
                        }

                        int total_width=0;
                	for (int col_index=first_col;col_index<=last_col;col_index++)
                        {
                        	s=header()->mapToSection(col_index);
                                // And now draw the boxes around the cells
				if (col_index==first_col) // if we're on the first col of the page draw also a hor line at the beginning
				{
                                	p.drawLine(Margin+total_width, Margin + yPos,
                                           	   Margin+total_width, Margin + yPos +fm.lineSpacing());
				}
                                // Lower hor. line
                                p.drawLine( Margin+total_width,  Margin + yPos+fm.lineSpacing(), Margin+total_width+(header()->sectionSize(s)),
                                            Margin + yPos+fm.lineSpacing());
                                // vertical line
                                p.drawLine(Margin+total_width+(header()->sectionSize(s)), Margin + yPos,
                                           Margin+total_width+(header()->sectionSize(s)),  Margin + yPos +fm.lineSpacing());
				// Test if the text is a number
				int align;
				QString text=(it.current())->text(s);
				// Test if it is a number
        			if (General::isNumber(text)) 
					align=ExpandTabs|AlignRight|AlignVCenter;
				else
					align=ExpandTabs;
                                p.drawText( Margin+total_width, Margin + yPos, (header()->sectionSize(s)),
                                            fm.lineSpacing(), align , text);
                                total_width=total_width+(header()->sectionSize(s));
                        }
                        yPos = yPos + fm.lineSpacing();
			rows_on_page++;
                }
                p.end();                                // send job to printer
        }
	delete printer;
	printer=0;
}


void ListView::saveData()
{
        QString filename=QFileDialog::getSaveFileName("", "*.txt", this );
        if ( !filename.isNull() )
        {
                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream ts(&f);

                	// save col heading
                	int s; // section
                	int col_index=0;
                	for (;col_index<columns();col_index++)
                	{
                        	s=header()->mapToSection(col_index);
                                ts << header()->label(s) << "\t";
                	}
			ts << endl << endl;

			// Save col data
                	QListViewItemIterator it( this );
                	// iterate through all items of the listview
			QString val;
                	for ( ; it.current(); ++it )
                	{
                                for (col_index=0;col_index<columns();col_index++)
                                {
                                        s=header()->mapToSection(col_index);
					val=(it.current())->text(s);
					if (val==QString::null)
                                 		ts << "(null)" << "\t";
					else
                                 		ts << val << "\t";
				}
				ts << endl;
                        }
                }
        }
}

