/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#ifndef LISTVIEW_H
#define LISTVIEW_H  

#include <qlistview.h>
#include <qprinter.h>

class row;

class ListView: public QListView
{
	Q_OBJECT
public:
	ListView(QWidget * parent=0, const QString& name=QString::null);
	//ListView(QWidget * parent=0, const char * name=0);
	virtual void printData(const QString& OutputFilename=QString::null, 
			       const QString& docName=QString::null, 
			       const QPrinter::Orientation& orient=QPrinter::Landscape);
	virtual void saveData();

protected:
	virtual void showEvent(QShowEvent * e);
	virtual void hideEvent(QHideEvent * e);

signals:
	// there is already a rightButtonPressed but it needs 
	// more params:  QListViewItem *, const QPoint &, int 
	// This one is needed for cases where we are only interested
	// in the position of the click
	// e.g. for showing context menus on right mouse click
        void rightButtonPressed(const QPoint & point);

	// idem for doubleClicked and returnPressed
	// They already exist but have QListViewItem argument
	// But we need them with row arguments
	void doubleClicked ( row * ); 
       	void returnPressed ( row * ); 

	void shown();
	void hidden();

private slots:
	// this is needed to emit a rightButtonPressed signal
	// with only a point argument
	void emitRightButtonPressed(QListViewItem * r, const QPoint & point, int col); 

	// THese are connect to the original signals
	// and emit the modified signals doubleClicked and returnPressed
	void emitDoubleClicked(QListViewItem * r);
	void emitReturnPressed(QListViewItem * r);
};


#endif  
