/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <qwindowsstyle.h>                                          
#include <qapplication.h>                                          
#include "qt_oraconnect.h"
#include "mdi.h"
#include "sqlcache.h"
#include "config.h"
#include <iostream.h> 
#include <stdlib.h> 
#include <qregexp.h> 
#include <qevent.h> 
#include <qmessagebox.h> 

////////////////////////////////
// Some global variables
///////////////////////////////
qt_oraconnect * gConnObj;
SQLCache* gSQLCache;
QString gUsername, gPassword, gTNSAlias, gCommandLineArg;
mdi * gMDI;
int gDefSheetWidth, gDefSheetHeight; 
ConfigFile * gConfigFile;

#ifdef INHOUSE
	const QString gVersion="1.3.0ih";
#else
	const QString gVersion="1.3.0";
#endif

////////////////////////////////////////////////////

int main( int argc, char ** argv )
{
	gConnObj=0;
	gSQLCache=0;
	gMDI=0;

	gUsername="";
	gPassword="";
	gTNSAlias="";
	gCommandLineArg="";

	// Create application object and filter Qt or X related command line arguments
	QApplication a( argc, argv ); 
	// Read config file
	QString ConfigFileName=getenv("DBACOMP_INI");
	if (!QFile(ConfigFileName).exists())
	{
		// Try to use a default when DBACOMP_INI is not set
		// or is incorrect
		cerr << "DBACOMP_INI not set or incorrect. Trying local dbacomp.ini file." << endl;
		ConfigFileName="dbacomp.ini";
	}
	if (QFile(ConfigFileName).exists())
	{
		gConfigFile=new ConfigFile(ConfigFileName);
		gConfigFile->read();

		gConfigFile->setGroup("GUI");
		gDefSheetWidth=gConfigFile->readEntry("sheet_width",600);
		gDefSheetHeight=gConfigFile->readEntry("sheet_height",400);
	}
	else
	{
		QMessageBox::critical( 0, "Initialisation error", "Initialisation file (e.g. dbacomp.ini) not found!\n\nUnsuccessfully tried environment variable DBACOMP_INI and\nlocal dbacomp.ini file. Set or correct DBACOMP_INI.\n\nDBAComp aborted!",QMessageBox::Ok | QMessageBox::Default,0,0);
		exit (1);
	}

	// Create the mdi window
	gMDI = new mdi();
	gConfigFile->setGroup("GUI");
	gMDI->resize(gConfigFile->readEntry("mdi_width",600),gConfigFile->readEntry("mdi_height",400));
	a.setMainWidget( gMDI );
	gMDI->show();

	// Check if there is a command line argument to logon
	if (argc==2) gCommandLineArg=argv[1];

	// to speed up testing
	//gCommandLineArg="system/pw@db";

	// We post a user event that will make the logon 
	// dialog show up after the MDI window has been shown
	QEvent* logonEvent = new QEvent(QEvent::User);
	a.postEvent(gMDI,logonEvent);
	// provoke a clean shutdown of the application
	// closing the mdi window and thus closing all windows and
	// disconnecting from the db
	QObject::connect( qApp, SIGNAL(aboutToQuit()), gMDI, SLOT(close()) );
	// Start main event loop
	return a.exec();
}
