/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "scriptdlg.h"
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <iostream.h>
#include <qfiledialog.h>
#include <qmessagebox.h>


ScriptDlg::ScriptDlg(const QString& fn, QWidget *parent, 
	 	     QString name, bool obj_choice)
        : QDialog( parent, name.local8Bit(), TRUE )
{
        QLabel *fl = new QLabel("Filename:", this);
        fl->setGeometry(10,40,90,20);
        filename = new QLineEdit(this);
        filename->setGeometry(100,40,200,20);
	filename->setText(fn);

	QPushButton *fb = new QPushButton("...",this);
	fb->setGeometry(298,35,30,30);
	connect( fb, SIGNAL(clicked()), SLOT(changeFile()) );

	if (obj_choice)
	{
        	QLabel *wl = new QLabel("Generate SQL for:", this);
        	wl->setGeometry(10,70,200,20);
	        what = new QButtonGroup(this);
       	 	what->setGeometry(100,90,200,60);
		QRadioButton * rb = new QRadioButton("Selected object",what);
		rb->setChecked(TRUE);
		rb->setGeometry(10,10,160,20);
		rb = new QRadioButton("All objects",what);
		rb->setGeometry(10,30,160,20);

		show_script = new QCheckBox("Show?", this);
		show_script->setChecked(TRUE);
        	show_script->setGeometry(10,150,70,20);

		spool = new QCheckBox("Spool?", this);
		spool->setChecked(FALSE);
        	spool->setGeometry(10,175,70,20);

		echo = new QCheckBox("Echo?", this);
		echo->setChecked(FALSE);
        	echo->setGeometry(10,200,70,20);

        	QPushButton *ok, *cancel;
        	ok = new QPushButton( "OK", this );
        	ok->setGeometry( 130,180, 80,30 );
        	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
        	cancel = new QPushButton( "Cancel", this );
        	cancel->setGeometry( 220,180, 80,30 );
        	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

		ok->setDefault(true);

		setCaption("Save SQL script as...");
		setFixedWidth(350);
		setFixedHeight(220);
	}
	else
        {
                show_script = new QCheckBox("Show?", this);
                show_script->setChecked(TRUE);
                show_script->setGeometry(10,70,70,20);

                QPushButton *ok, *cancel;
                ok = new QPushButton( "OK", this );
                ok->setGeometry( 130,100, 80,30 );
                connect( ok, SIGNAL(clicked()), SLOT(accept()) );
                cancel = new QPushButton( "Cancel", this );
                cancel->setGeometry( 220,100, 80,30 );
                connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

                ok->setDefault(true);

                setCaption("Save SQL script as...");
                setFixedWidth(350);
                setFixedHeight(140);
        }
}

ScriptDlg::~ScriptDlg()
{
#ifdef DESTRACE
	cerr << "ScriptDlg destructor" << endl;
#endif
}

const QString ScriptDlg::getFilename() const
{
	return filename->text();
}

bool ScriptDlg::getAllObj() const
{
	if (what->id(what->selected())==0)
		return false;
	else
		return true;
}

bool ScriptDlg::getShow() const
{
	return show_script->isChecked();
}

bool ScriptDlg::getSpool() const
{
	return spool->isChecked();
}

bool ScriptDlg::getEchoOn() const
{
	return echo->isChecked();
}

void ScriptDlg::changeFile()
{
    QString s = QFileDialog::getSaveFileName( filename->text(), "*.sql", this );
    if ( !s.isNull() ) 
	{
		filename->setText(s);
    } 
}

void ScriptDlg::accept()
{
	if ((QDir (QFileInfo(filename->text()).dirPath()).exists())
	&& !(QFile(QFileInfo(filename->text()).filePath()).exists()))
		QDialog::accept();
	else if (!(QDir (QFileInfo(filename->text()).dirPath()).exists()))
		QMessageBox::critical( 0, "File save error", QString("The directory ")+QString(QFileInfo(filename->text()).dirPath())+QString(" does not exist!"),QMessageBox::Ok | QMessageBox::Default,0,0);
	else // File already exists
		// check if user has the right to overwrite the file
		if (!(QFileInfo(filename->text()).isWritable()))
			QMessageBox::critical( 0, "File save error", QString("The file ")+QString(QFileInfo(filename->text()).filePath())+QString(" already exists and you don't have any rights to overwrite it!\n\nPlease change the name or directory of the file."),QMessageBox::Ok | QMessageBox::Default,0,0);
		else //you have the right to overwrite the file
			if (QMessageBox::warning( 0, "File save warning", QString("The file ")+QString(QFileInfo(filename->text()).filePath())+QString(" already exists!\n\nDo you want to overwrite?"),QMessageBox::Yes,QMessageBox::No | QMessageBox::Default,0)==QMessageBox::Yes)
				QDialog::accept();
}
