/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <stdlib.h>
#include "sqlcache.h"
// For qt3
#include <qregexp.h>

SQLCache::SQLCache(const QString& fn, const QString oraver, const int maxitems)
	: filename(fn), oraversion(oraver), stream(0), file(fn) 
{
	//cache = new QCache<QString>(maxitems,17,false);
	cache = new QCache<QString>(maxitems,101,false);
	cache->setAutoDelete(true);
}

SQLCache::~SQLCache()
{
#ifdef DESTRACE
	cerr << "SQLCache destructor" << endl;
#endif
	delete stream;
	stream=0;
	delete cache;
	cache=0;
}

const QString SQLCache::getSQL(const QString& name) 
{
#ifdef SQLTRACE
	cerr << "SQL name: " << name << endl;
#endif

	QString* sql = cache->find(name,true);
	//sql = cache->find(name,true);
	if (sql)
	{
		return *sql;
	}
	else
	{
		QString sqlfromfile=getSQLfromFile(name,oraversion);
		if (sqlfromfile!="") 
		{
			// new QString(sqlfromfile) is necessary
			// to create a copy of the sqlfromfile 
			// that will not be deleted when sqlfromfile
			// goes out of scope
			QString * s = new QString(sqlfromfile);
			if (!cache->insert(name,s))
			{
				// I don't know what could make it fail
				// but the doc says it could
				// and in that case we need to delete
				// the string
				cerr << "Cache insert failed!" << endl;
				delete s;
				s=0;
			}
		}
		return sqlfromfile;
	}
	//return getSQLfromFile(name,oraversion);
}

QString SQLCache::getSQLfromFile(const QString& name, const QString& section) 
{
	if ( file.open(IO_ReadOnly) )
	{
		// file opened successfully
		stream = new QTextStream( &file );        // use a text stream
		QString s="";
		QString l="";
		// search for the section in the file
		// corresponding to the specified section
		QRegExp r;
		r.setCaseSensitive(false);
		r.setWildcard(false);
		while ( !stream->eof() )
		{        // until end of file...
                        l = stream->readLine();  // line of text excluding '\n'
			// ignore comments
			if (l.left(1)=="#") continue;
			else s=l;
			//s = stream->readLine();  // line of text excluding '\n'
			s = s.simplifyWhiteSpace();
			s = s.replace(QRegExp(" "),"");
			if (s=="["+section+"]")
			{
				break;
			}
		}

		if (stream->eof())
		{
        		// Unwind the waitCursors that may have been set elsewhere
        		while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
 			QMessageBox::critical( 0, "Error in SQL cache file ["+filename+"]", "No section corresponding to Oracle version " +section+ " could be found! \nDBAComp aborted!",QMessageBox::Ok | QMessageBox::Default,0,0);
            		file.close();
            		delete stream;
            		stream=0;
			exit (1);
		}

		// In this section look for the name
		while ( !stream->eof() )
                {        // until end of file...

                        l = stream->readLine();  // line of text excluding '\n'
			// ignore comments
			if (l.left(1)=="#") continue;
			else s=l;
                        //s = stream->readLine();  // line of text excluding '\n'
                        s = s.simplifyWhiteSpace();
                        r = QRegExp("^\\[[0-9][0-9]*\\.[0-9]\\]");
			// Before
                        //r = "^\\[[0-9][0-9]*\\.[0-9]\\]";
			// if we get to another section of Oracle then quit
			// This means the SQL is not in this section
			if (s.contains(r)>0)
			{	
				file.close();
				delete stream;
				stream=0;
                                return "";
			}
			else
			{
				// Test if the name is in the current line
				r="^"+name+":";
				if (s.contains(r) > 0)
				{
					// extract the sql statement and return it
					int label_len=name.length()+1;
					// check if line is continued
					QString sqlstring=s.mid(label_len,s.length()-label_len);
					// As long as there is a continuation char.
					while (s.right(1)=="\\")
					{
                        			l = stream->readLine();  // line of text excluding '\n'
						// ignore comments
						if (l.left(1)=="#") continue;
						else s=l;
						//s = stream->readLine();  
						s = s.simplifyWhiteSpace();

						// remove trailing "\"
						sqlstring = sqlstring.left(sqlstring.length()-1);
						sqlstring=sqlstring+" "+s;
						//if (!stream->eof()) 
						//	sqlstring=sqlstring+" "+s;
						//else
						//	break;
						if (stream->eof()) break;
					}

					file.close();
					delete stream;
					stream=0;
					return sqlstring.simplifyWhiteSpace();
				}
			}
				
                }
	file.close();
	delete stream;
	stream=0;
	return "";
        }
	else
	{
		QMessageBox::critical( 0, "SQL Cache file error", "SQL Cache file ("+filename+") could not be found!\n\nUnsuccessfully tried 'sql_cache_file_prefix' entry and local dbacomp."+oraversion+".sql file.\nSet or correct the 'sql_cache_file_prefix' entry in the 'PATH' section of the dbacomp.ini file.\n\nDBAComp aborted!",QMessageBox::Ok | QMessageBox::Default,0,0);
		exit(1);
		return "";
	}
}

void SQLCache::clear()
{
	cache->clear();
}
