/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "tabdatadisplay.h" 
#include "griddatadisplay.h" 
#include "formdatadisplay.h" 
#include "mledatadisplay.h" 
#include "datadisplay.h" 
#include <qpopupmenu.h> 
#include <iostream.h> 

//TabDataDisplay::TabDataDisplay(qt_oraconnect * cObj, QWidget * parent, const char * name) 
TabDataDisplay::TabDataDisplay(qt_oraconnect * cObj, QWidget * parent, 
			       const QString& name) 
	:TabView(parent,name), DataDisplay(cObj,0)
{
	connect(this, SIGNAL(selected(const QString &)), this, SLOT(setCurrentTab(const QString & )));
}

TabDataDisplay::~TabDataDisplay()
{
#ifdef DESTRACE
	cerr << "TabDataDisplay destructor" << endl;
#endif
}

void TabDataDisplay::addTab(QWidget *widget, const QString& name)
{
	TabView::addTab(widget,name);
	currentTab=name;

	TabContent* tc = new TabContent;
	tc->isDD=false;
	tc->widget=widget;
	tc->dd=0;
	TabContents.insert(name,tc);
}

void TabDataDisplay::addGridTab(GridDataDisplay *grid, const QString& name)
{
        TabView::addTab(grid,name);
        currentTab=name;

        TabContent* tc = new TabContent;
        tc->isDD=true;
        tc->widget=grid;
        tc->dd=grid;
        TabContents.insert(name,tc);
	connect(grid, SIGNAL(rightButtonPressed(const QPoint &)), 
		this, SIGNAL(rightButtonPressed(const QPoint &)));
	connect(grid, SIGNAL(SQLError(const QString&)), 
		this, SIGNAL(SQLError(const QString&)));
}

void TabDataDisplay::addGridTab(QString sql, const QString& name, bool ana)
{
	GridDataDisplay *g = new GridDataDisplay(getConnObj(),sql,this,name,ana);
	g->setAllColumnsShowFocus(TRUE);
	connect(g, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
	addGridTab(g,name);
}

void TabDataDisplay::addFormTab(FormDataDisplay *form, const QString& name)
{
        TabView::addTab(form,name);
        currentTab=name;

        TabContent* tc = new TabContent;
        tc->isDD=true;
        tc->widget=form;
        tc->dd=form;
        TabContents.insert(name,tc);
	connect(form, SIGNAL(rightButtonPressed(const QPoint &)), this, SIGNAL(rightButtonPressed(const QPoint &)));
	connect(form, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
}

void TabDataDisplay::addFormTab(QString sql, const QString& name, int cols, int lw, int tw, QString SingleField,int sfw, int sfh)
{
	FormDataDisplay *f = new FormDataDisplay(getConnObj(),sql,cols,lw,tw,this,name,SingleField,sfw,sfh);
	connect(f, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
	addFormTab(f,name);
}

void TabDataDisplay::addMLETab(MLEDataDisplay *mle, const QString& name)
{
        TabView::addTab(mle,name);
        currentTab=name;

        TabContent* tc = new TabContent;
        tc->isDD=true;
        tc->widget=mle;
        tc->dd=mle;
        TabContents.insert(name,tc);

        connect(mle, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
}

void TabDataDisplay::addMLETab(QString sql, const QString& name, bool headers, 
			       QString line_sep, QString col_sep, QString head_sep)
{
        MLEDataDisplay *m = new MLEDataDisplay(getConnObj(),sql,this,name,headers,line_sep, col_sep, head_sep);
        connect(m, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
        addMLETab(m,name);
}

void TabDataDisplay::addTabTab(TabDataDisplay *tab, const QString& name)
{
        TabView::addTab(tab,name);
        currentTab=name;

        TabContent* tc = new TabContent;
        tc->isDD=true;
        tc->widget=tab;
        tc->dd=tab;
        TabContents.insert(name,tc);
	connect(tab, SIGNAL(selected(const QString &)), this, SLOT(translateSubtabNameToTabName(const QString &)));
	connect(tab, SIGNAL(rightButtonPressed(const QPoint &)), this, SIGNAL(rightButtonPressed(const QPoint &)));
	connect(tab, SIGNAL(SQLError(const QString&)), this, SIGNAL(SQLError(const QString&)));
}

void TabDataDisplay::translateSubtabNameToTabName(const QString & SubTabName)
{
	emit selected(currentTab);
}

long TabDataDisplay::retrieve(const ObjParm * parm)
{
	if (currentTab)
	{
		if (TabContents[currentTab]->isDD)
		{
			TabContents[currentTab]->dd->setNoInitQuery(getNoInitQuery());
			return ((DataDisplay*)TabContents[currentTab]->dd)->retrieve(parm);
		}
		else
			return(0);
	}
	else
		return 0;
}

QPopupMenu* TabDataDisplay::getMenuAddition()
{
        if (currentTab)
        {
                if (TabContents[currentTab]->isDD)
		{
			QPopupMenu* menu=((DataDisplay*)TabContents[currentTab]->dd)->getMenuAddition();
			// set a menu name (which can be used for the caption
			// of the menu item )
			if (menu) menu->setName(currentTab.local8Bit());
			return menu;
		}
                else
                        return(0);
        }
        else
                return 0;
}


void TabDataDisplay::setSort(const QString& s)
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        ((DataDisplay*)TabContents[currentTab]->dd)->setSort(s);
}

void TabDataDisplay::setFilter(const QString& s)
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        ((DataDisplay*)TabContents[currentTab]->dd)->setFilter(s);
}

const QString& TabDataDisplay::getSort() const
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        return ((DataDisplay*)TabContents[currentTab]->dd)->getSort();
				else return QString::null;
		else return QString::null;

}
 
const QString& TabDataDisplay::getFilter() const
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        return ((DataDisplay*)TabContents[currentTab]->dd)->getFilter();
                else return QString::null;
        else return QString::null;                                                           
}                                                                                       

//const char* TabDataDisplay::getSQL()
const QString& TabDataDisplay::getSQL()
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        return ((DataDisplay*)TabContents[currentTab]->dd)->getSQL();
                else return QString::null;
        else return QString::null;
}

void TabDataDisplay::setCurrentTab(const QString & name )
{
	currentTab=name;
}

const QStringList& TabDataDisplay::getColList() const 
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        return ((DataDisplay*)TabContents[currentTab]->dd)->getColList();
		else
			return getColList();
	else
		return getColList();
}

const QString TabDataDisplay::getCurrentTab()
{
	return currentTab;
}

void TabDataDisplay::print()
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        ((DataDisplay*)TabContents[currentTab]->dd)->print();
}

void TabDataDisplay::save()
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        ((DataDisplay*)TabContents[currentTab]->dd)->save();
}

void TabDataDisplay::setLogMode(const bool ror)
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        ((DataDisplay*)TabContents[currentTab]->dd)->setLogMode(ror);
}

const bool TabDataDisplay::getLogMode() const
{
        if (currentTab)
                if (TabContents[currentTab]->isDD)
                        return ((DataDisplay*)TabContents[currentTab]->dd)->getLogMode();
		else return(false);
	else return(false);
}

//void TabDataDisplay::setNoInitQuery(const bool niq)
//{
//	DataContainer::setNoInitQuery(niq);
//cerr << "setNoInitQuery" << endl;
//cerr << "niq:" << niq << endl;
//	QDictIterator<TabContent> tabIterator(TabContents);
//	while (tabIterator.current())
//	{
//		if (tabIterator.current()->isDD)
//		{
//cerr << "setting niq" << endl;
//			tabIterator.current()->dd->setNoInitQuery(niq);
//		}
//		++tabIterator;
//	}
//}
