/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <qkeycode.h> 
#include "usersqllist.h" 
#include "row.h" 
#include "dblist.h" 
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <iostream.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qfiledialog.h>

#include "refresh.xpm" 

#include "config.h"
extern ConfigFile *  gConfigFile;

//UserSQLList::UserSQLList(QWidget * parent, const char * name)
UserSQLList::UserSQLList(QWidget * parent, const QString& name)
       : Sheet(parent, name),lv(0)
{
	// set the action menu
	fillActionMenu();
    	gConfigFile->setGroup("PATH");
	setFilename(gConfigFile->readEntry("user_sql",""));
        if (!QFile(filename).exists())
        {
                cerr << "user_sql not set or incorrect. Trying local user.sql." << endl;
                setFilename("user.sql");
        }

	setCaption("User defined SQL ["+filename+"]");
	setupSQLList();
}

void UserSQLList::setFilename(const QString& fn)
{
	filename=fn;
}

void UserSQLList::setupSQLList()
{
	lv = new ListView(this);
	setCentralWidget(lv);
	lv->addColumn("Name");
	lv->addColumn("SQL");
	lv->setSorting(0);
	lv->setAllColumnsShowFocus(TRUE);
    	connect(lv, SIGNAL(doubleClicked(row*)), this, SLOT(callInfo(row *)));
    	connect(lv, SIGNAL(returnPressed(row*)), this, SLOT(callInfo(row *)));
    	connect(lv, SIGNAL(rightButtonPressed(const QPoint &)), this, SLOT(ContextMenu(const QPoint &)));                                                                                                        
	retrieve();
}

void UserSQLList::callInfo(row * r)
{
	DBList * w = new DBList(r->getCellbyName("SQL"),0,"User defined SQL: "+r->getCellbyName("Name"),true);
	w->show();
	
}

void UserSQLList::fillActionMenu()
{
        // set the action menu
        getActionMenu()->insertItem( "Change SQL file",this,SLOT(changeFile()),CTRL+Key_O);
        QPixmap refreshIcon ;
        refreshIcon = QPixmap( refresh );
        getActionMenu()->insertItem(refreshIcon, "Refresh",this,SLOT(retrieve()),CTRL+Key_R);

	setupActionToolBar();

}

QPopupMenu * UserSQLList::addGenerateSQLToActionMenu()
// This can be called when a descendant needs it
// It then only must add its own menus to the Generate SQL menu
{
        QPopupMenu * action_menu = getActionMenu();
        action_menu->insertSeparator();

        QPopupMenu * generate_menu = new QPopupMenu(this);
        action_menu->insertItem("Generate SQL",generate_menu);
	return generate_menu;
}

void UserSQLList::setupActionToolBar()
{
	QToolBar* tb = getActionToolBar();

        QPixmap refreshIcon ;
        refreshIcon = QPixmap( refresh );
        QToolButton * refresh = new QToolButton( refreshIcon, "Refresh", 0,
                                            this, SLOT(retrieve()),
                                            tb, "Refresh" );
        refresh->show();
}


UserSQLList::~UserSQLList()
{
#ifdef DESTRACE
	cerr << "UserSQLList destructor" << endl;
#endif
}

void UserSQLList::retrieve()
{
	int recs=0;
	row * lastrow=0;
	lv->clear();
	QFile file(filename);
	if ( file.open(IO_ReadOnly) )
	{
		// file opened successfully
		QTextStream * stream = new QTextStream( &file );
		QString s="";

		// look for the name
		while ( !stream->eof() )
		{        // until end of file...
			s = stream->readLine();  // line of text excluding '\n'
			s = s.simplifyWhiteSpace();
			// extract the name of the SQL
			int colon_pos=s.find(":");
			if (colon_pos!= -1)
			{
				recs++;
				QString name=s.left(colon_pos);
				// extract the sql statement and return it
				int label_len=name.length()+1;
				// check if line is continued
				QString sqlstring=s.mid(label_len,s.length()-label_len);
				while (s.right(1)=="\\")
				{
					s = stream->readLine();  
					s = s.simplifyWhiteSpace();

					// remove trailing "\"
					sqlstring = sqlstring.left(sqlstring.length()-1);
					sqlstring=sqlstring+" "+s;
					//if (!stream->eof()) 
					//	sqlstring=sqlstring+" "+s;
					//else
					//	break;
					if (stream->eof()) break;
				}

				row * r = new row(lv,lastrow);
				lastrow=r;
				r->createCells(2);
				r->setText(0,name);
				r->setText(1,sqlstring.simplifyWhiteSpace());
			}
		}
				
		file.close();
		delete stream;
		stream=0;
		lv->setSelected(lv->firstChild(),true);
	}
	else
	{
		QMessageBox::critical( 0, "User SQL error", "User SQL file ("+filename+") could not be found!\n\nUnsuccessfully tried 'user_sql' entry and local user.sql file.\nSet or correct the 'user_sql' entry in the 'PATH' section of the dbacomp.ini file and restart DBAComp.\n\nThe rest of DBAComp will be unaffected by this problem!",QMessageBox::Ok | QMessageBox::Default,0,0);
	}

    	QString msg;
    	msg.setNum(recs);

	if (recs==1)
    	statusBar()->message(msg+" record found.");
	else
    	statusBar()->message(msg+" records retrieved.");
}

void UserSQLList::changeFile()
{

	QString s(QFileDialog::getOpenFileName(QString::null,"*.sql",this));
    	if (s.isEmpty())
        	return;
	else
	{
		setFilename(s);
		setCaption("User defined SQL ["+filename+"]");
		retrieve();
	}
}

void UserSQLList::Print()
{
        lv->printData();
}
