/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "w_dba_roles.h"
#include "w_dba_roles_info.h"

#include <stdlib.h>
#include <qmessagebox.h>
#include "scriptdlg.h"
#include "gensql.h"
#include "editor.h"
#include "config.h"
#include <qfile.h>
#include <qtextstream.h>

w_dba_roles::w_dba_roles()
	: DBList("dba_roles",0,"DBA_ROLES")
{
	addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_roles::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Create...",this,SLOT(generate_role_create()));
        m->insertItem("Drop...",this,SLOT(generate_role_drop()));
        m->insertItem("Grant to...",this,SLOT(generate_role_grant()));
        m->insertSeparator();
        m->insertItem("All privileges...",this,SLOT(generate_all_privs()));
        m->insertItem("Table privileges...",this,SLOT(generate_tab_privs()));
        m->insertItem("Column privileges...",this,SLOT(generate_col_privs()));
        m->insertItem("System privileges...",this,SLOT(generate_sys_privs()));
        m->insertItem("Role privileges...",this,SLOT(generate_role_privs()));
        return m;
}

void w_dba_roles::callInfo(row * clickedRow)
{
	w_dba_roles_info *wui = new w_dba_roles_info(clickedRow->getCellbyName("role"));
}

void w_dba_roles::generate_tab_privs()
{
        generate_sql("tab_privs.sql","tab_privs");
}

void w_dba_roles::generate_role_privs()
{
        generate_sql("role_privs.sql","role_privs");
}

void w_dba_roles::generate_sys_privs()
{
        generate_sql("sys_privs.sql","sys_privs");
}

void w_dba_roles::generate_col_privs()
{
        generate_sql("col_privs.sql","col_privs");
}

void w_dba_roles::generate_all_privs()
{
        generate_sql("all_privs.sql","all_privs");
}

void w_dba_roles::generate_role_create()
{
        generate_sql("role_create.sql","role_create");
}

void w_dba_roles::generate_role_drop()
{
        generate_sql("role_drop.sql","role_drop");
}

void w_dba_roles::generate_role_grant()
{
        generate_sql("role_grant.sql","role_grant");
}

void w_dba_roles::generate_sql(const QString& fn, const QString& what)
{
        QString object_name,filename,owner;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("role");
                                int retval;
                                if (what=="tab_privs")
                                        retval=GenSQL::RoleTabPrivs(t,object_name);
                                else if (what=="role_privs")
                                        retval=GenSQL::RoleRolePrivs(t,object_name);
                                else if (what=="sys_privs")
                                        retval=GenSQL::RoleSysPrivs(t,object_name);
                                else if (what=="col_privs")
                                        retval=GenSQL::RoleColPrivs(t,object_name);
                                else if (what=="all_privs")
                                        retval=GenSQL::RolePrivs(t,object_name);
                                else if (what=="role_grant")
                                        retval=GenSQL::RoleGrant(t,object_name);
                                else if (what=="role_create")
				{
                                	QString pw=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("password_required");
                                        retval=GenSQL::RoleCreate(t,object_name,pw);
				}
                                else if (what=="role_drop")
                                        retval=GenSQL::RoleDrop(t,object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("role");
                                        int retval;
                                        if (what=="tab_privs")
                                                retval=GenSQL::RoleTabPrivs(t,object_name);
                                        else if (what=="role_privs")
                                                retval=GenSQL::RoleRolePrivs(t,object_name);
                                        else if (what=="sys_privs")
                                                retval=GenSQL::RoleSysPrivs(t,object_name);
                                        else if (what=="col_privs")
                                                retval=GenSQL::RoleColPrivs(t,object_name);
                                        else if (what=="all_privs")
                                                retval=GenSQL::RolePrivs(t,object_name);
                                        else if (what=="role_grant")
                                                retval=GenSQL::RoleGrant(t,object_name);
                                        else if (what=="role_create")
					{
                                		QString pw=((row *)it.current())->getCellbyName("password_required");
                                                retval=GenSQL::RoleCreate(t,object_name,pw);
					}
                                        else if (what=="role_drop")
                                                retval=GenSQL::RoleDrop(t,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

