/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "w_dba_rollback_segs.h"
#include "w_dba_rollback_segs_info.h"

#include "scriptdlg.h"
#include "editor.h"
#include "gensql.h"
#include "config.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <stdlib.h>

w_dba_rollback_segs::w_dba_rollback_segs()
	: DBList("dba_rollback_segs",0,"DBA_ROLLBACK_SEGS")
{
	addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_rollback_segs::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Create...",this,SLOT(generate_rbs_create()));
        m->insertItem("Drop...",this,SLOT(generate_rbs_drop()));
        m->insertItem("Online...",this,SLOT(generate_rbs_online()));
        m->insertItem("Offline...",this,SLOT(generate_rbs_offline()));
        m->insertItem("Shrink...",this,SLOT(generate_rbs_shrink()));
        return m;
}

void w_dba_rollback_segs::callInfo(row * clickedRow)
{
	w_dba_rollback_segs_info *wui = new w_dba_rollback_segs_info(clickedRow->getCellbyName("segment_id"),clickedRow->getCellbyName("segment_name"));
}

void w_dba_rollback_segs::generate_rbs_create()
{
        generate_sql("rbs_create.sql","rbs_create");
}

void w_dba_rollback_segs::generate_rbs_drop()
{
        generate_sql("rbs_drop.sql","rbs_drop");
}

void w_dba_rollback_segs::generate_rbs_online()
{
        generate_sql("rbs_online.sql","rbs_online");
}

void w_dba_rollback_segs::generate_rbs_offline()
{
        generate_sql("rbs_offline.sql","rbs_offline");
}

void w_dba_rollback_segs::generate_rbs_shrink()
{
        generate_sql("rbs_shrink.sql","rbs_shrink");
}

void w_dba_rollback_segs::generate_sql(const QString& fn, const QString& what)
{
        QString object_name,filename,owner;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("segment_name");
                                int retval;
                                if (what=="rbs_create")
                                        retval=GenSQL::RBSCreate(t,object_name);
                                else if (what=="rbs_drop")
                                        retval=GenSQL::RBSDrop(t,object_name);
                                else if (what=="rbs_online")
                                        retval=GenSQL::RBSOnline(t,object_name);
                                else if (what=="rbs_offline")
                                        retval=GenSQL::RBSOffline(t,object_name);
                                else if (what=="rbs_shrink")
                                        retval=GenSQL::RBSShrink(t,object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("segment_name");
                                        int retval;
                                        if (what=="rbs_create")
                                                retval=GenSQL::RBSCreate(t,object_name);
                                        else if (what=="rbs_drop")
                                                retval=GenSQL::RBSDrop(t,object_name);
                                        else if (what=="rbs_online")
                                                retval=GenSQL::RBSOnline(t,object_name);
                                        else if (what=="rbs_offline")
                                                retval=GenSQL::RBSOffline(t,object_name);
                                        else if (what=="rbs_shrink")
                                                retval=GenSQL::RBSShrink(t,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

