/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include "config.h"
#include "griddatadisplay.h"
#include "w_dba_tables_info.h"
#include "w_dba_indexes_info.h"
#include "w_dba_constraints_info.h"
#include "sqlcache.h"
#include "row.h"

w_dba_tables_info::w_dba_tables_info(const ObjParm& parm)
	: DBTab(parm,0,"Table information for "+parm.getParm(0)+"."+parm.getParm(1))
{
	//setCaption("Table information for "+parm.getParm(0)+"."+parm.getParm(1));
	makeTabs();
        show();
}

void w_dba_tables_info::makeTabs()
{
	addFormTab("dba_objects_per_table","Object",2,130,130);
	addGridTab("child_tables","FK Children");
	addFormTab("dba_segments_per_table","Segment",2,130);

	gConfigFile->setGroup("MISC");
	QString max_rows = gConfigFile->readEntry("max_rows","1000000000000");	
	GridDataDisplay * rowtab = new GridDataDisplay(gConnObj,"select * from "+getWinParm()->getParm(0)+"."+getWinParm()->getParm(1) + " where rownum < "+max_rows,0,"Rows");
	addGridTab(rowtab,"Rows (limited to "+max_rows+")","dynamically generated");

	//addGridTab("dba_constraints_per_table","Constraints");
	// Make a grid tab that can call another window (for the constraints)
	GridDataDisplay * constab = new GridDataDisplay(gConnObj,gSQLCache->getSQL("dba_constraints_per_table"),0,"Constraints");
        connect(constab, SIGNAL(doubleClicked(row*)), this, SLOT(callConstraintInfo(row *)));
        connect(constab, SIGNAL(returnPressed(row*)), this, SLOT(callConstraintInfo(row *)));
	addGridTab(constab,"Constraints","dba_constraints_per_table");

	// Make a grid tab that can call another window (for the indexes)
	GridDataDisplay * indtab = new GridDataDisplay(gConnObj,gSQLCache->getSQL("dba_indexes_per_table"),0,"Indexes");
        connect(indtab, SIGNAL(doubleClicked(row*)), this, SLOT(callIndexInfo(row *)));
        connect(indtab, SIGNAL(returnPressed(row*)), this, SLOT(callIndexInfo(row *)));
	addGridTab(indtab,"Indexes","dba_indexes_per_table");

	addGridTab("dba_tab_columns_per_table","Columns");
	addGridTab("dba_triggers_per_table","Triggers");
	addGridTab("dba_tab_privs_per_table","Privs");
	addGridTab("dba_tab_partitions_per_table","Partitions");
	addGridTab("dba_tab_subpartitions_per_table","Subpartitions");
	addFormTab("dba_tab_comments_per_table","Comments",1,100,20,"COMMENTS",400,200);
	addGridTab("dba_lobs_per_table","Lobs");
}

void w_dba_tables_info::callIndexInfo(row * clickedRow)
{
        ObjParm parm;
        parm.addParm(getWinParm()->getParm(0)); // username
        parm.addParm(clickedRow->getCellbyName("index_name"));
        w_dba_indexes_info *w = new w_dba_indexes_info(parm);
}

void w_dba_tables_info::callConstraintInfo(row * clickedRow)
{
        ObjParm parm;
        parm.addParm(getWinParm()->getParm(0)); // username
        parm.addParm(clickedRow->getCellbyName("constraint_name"));
        w_dba_constraints_info *w = new w_dba_constraints_info(parm);
}
